/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#include <time.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include "merc.h"
#include "tables.h"
#include "lookup.h"

/* command procedures needed */
DECLARE_DO_FUN(do_look		);
DECLARE_DO_FUN(do_recall	);
DECLARE_DO_FUN(do_stand		);
DECLARE_DO_FUN(do_dismount	);

char * const dir_name[] =
{
    "north", "east", "south", "west", "up", "down", "northeast", "southeast",
    "southwest", "northwest", "clockwise", "counterclockwise"
};

const int trap_door[] =
{
    TRAP_N, TRAP_E, TRAP_S, TRAP_W, TRAP_U, TRAP_D, TRAP_NE, TRAP_SE, TRAP_SW,
    TRAP_NW, TRAP_CW, TRAP_CCW
};

const sh_int rev_dir[] =
{
    2, 3, 0, 1, 5, 4, 8, 9, 6, 7, 11, 10
};

const sh_int movement_loss[SECT_MAX] =
{
    1, 2, 2, 3, 4, 6, 8, 2, 6, 10, 6, 14, 14
};



/*
 * Local functions.
 */
int	find_door	args( ( CHAR_DATA *ch, char *arg ) );
bool	has_key		args( ( CHAR_DATA *ch, int key ) );



void move_char( CHAR_DATA *ch, int door, bool follow )
{
    CHAR_DATA *fch, *fch_next;
    ROOM_INDEX_DATA *in_room;
    ROOM_INDEX_DATA *to_room;
    EXIT_DATA *pexit;

    if ( door < 0 || door > 11 )
    {
	bug( "Do_move: bad door %d.", door );
	return;
    }

    /*
     * Exit trigger, if activated, bail out. Only PCs are triggered.
     */
    if ( !IS_NPC(ch) && mp_exit_trigger(ch, door) )
	return;
    if ( !CHAR_KILLED(ch) && !IS_NPC(ch) && rp_exit_trigger(ch, door) )
	return;

    in_room = ch->in_room;
    if ( (pexit   = in_room->exit[door]) == NULL
    ||   (to_room = pexit->u1.to_room) == NULL 
    ||    !can_see_room(ch, pexit->u1.to_room) )
    {
	send_to_char( "Alas, you cannot go that way.\n\r", ch );
	return;
    }
    
    if ( ch->riding != NULL )
    {
	if ( IS_AFFECTED(ch->riding, AFF_PARALYSIS) )
	{
	    send_to_char( "Your mount wont budge!\n\r", ch );
	    return;
	}
    }
    
    if ( ch->riding != NULL && ch->riding->position <= POS_SITTING )
    {
	send_to_char( "Your mount needs to stand first.", ch );
	return;
    }

    if (IS_SET(pexit->exit_info, EX_CLOSED)
    &&  (!IS_AFFECTED(ch, AFF_PASS_DOOR) || IS_SET(pexit->exit_info,EX_NOPASS))
    &&   !IS_TRUSTED(ch,ANGEL))
    {
	act( "The $d is closed.", ch, NULL, pexit->keyword, TO_CHAR );
	return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM)
    &&   ch->master != NULL
    &&   in_room == ch->master->in_room )
    {
	send_to_char( "What?  And leave your beloved master?\n\r", ch );
	return;
    }

    if ( !is_room_owner(ch,to_room) && room_is_private( to_room ) )
    {
	send_to_char( "That room is private right now.\n\r", ch );
	return;
    }

    if ( !IS_NPC(ch) )
    {
	int iClass, iGuild;
	bool flying;
	int move;

	for ( iClass = 0; iClass < MAX_CLASS; iClass++ )
	{
	    for ( iGuild = 0; iGuild < MAX_GUILD; iGuild ++)	
	    {
	    	if ( iClass != ch->class
	    	&&   to_room->vnum == class_table[iClass].guild[iGuild] )
	    	{
		    /* Andaras - modified to allow sub-classing. */
		    bool found = FALSE;
		    int i;
		    
		    for ( i=0; i<MAX_GUILD; i++ )
		    {
			if ( class_table[ch->class].guild[i] == to_room->vnum )
			{
			    found = TRUE;
			    break;
			}
		    }
		    
		    if ( !found )
		    {
		    send_to_char( "You aren't allowed in there.\n\r", ch );
		    return;
		    }
		}
	    }
	}

	if ( ch->riding == NULL )
	    flying = IS_AFFECTED( ch, AFF_FLYING ) != 0;
	else
	    flying = IS_AFFECTED( ch->riding, AFF_FLYING ) != 0;

	if ( in_room->sector_type == SECT_AIR
	||   to_room->sector_type == SECT_AIR )
	{
	    if ( !flying && !IS_IMMORTAL(ch) )
	    {
		send_to_char( "You can't fly.\n\r", ch );
		return;
	    }
	}

	if ( (in_room->sector_type == SECT_WATER_NOSWIM
	||    to_room->sector_type == SECT_WATER_NOSWIM)
  	&&    !flying )
	{
	    OBJ_DATA *obj;
	    bool found;

	    /*
	     * Look for a boat.
	     */
	    found = FALSE;

	    if (IS_IMMORTAL(ch))
		found = TRUE;

	    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
	    {
		if ( obj->item_type == ITEM_BOAT )
		{
		    found = TRUE;
		    break;
		}
	    }
	    if ( !found )
	    {
		send_to_char( "You need a boat to go there.\n\r", ch );
		return;
	    }
	}

	move = movement_loss[UMIN(SECT_MAX-1, in_room->sector_type)]
	     + movement_loss[UMIN(SECT_MAX-1, to_room->sector_type)];
        move /= 2;  /* i.e. the average */

	/* conditional effects */
	if ( (IS_AFFECTED(ch, AFF_FLYING)
	&&    in_room->sector_type != SECT_UNDERWATER
	&&    in_room->sector_type != SECT_UNDERWATER_BOTTOM
	&&    to_room->sector_type != SECT_UNDERWATER
	&&    to_room->sector_type != SECT_UNDERWATER_BOTTOM)
	||   IS_AFFECTED(ch, AFF_HASTE) )
	    move /= 2;
	if ( IS_AFFECTED(ch, AFF_SLOW))
	    move *= 2;
	if ( IS_AFFECTED(ch, AFF2_WATERS_WAYS)
	&&   (in_room->sector_type == SECT_UNDERWATER
	||    in_room->sector_type == SECT_UNDERWATER_BOTTOM
	||    to_room->sector_type == SECT_UNDERWATER
	||    to_room->sector_type == SECT_UNDERWATER_BOTTOM) )
	    move /= 2;

	if ( ch->riding != NULL )
	    move /= ( ch->riding->level / 10 ) + 1;
        else if ( has_customization(ch, DISADVANTAGE_LAME) )
            move *= 2;

	if ( ch->move < move )
	{
	    send_to_char( "You are too exhausted.\n\r", ch );
	    return;
	}

	WAIT_STATE( ch, 1 );
	ch->move -= move;
    }
    
    if ( IS_SET(to_room->room_flags, ROOM_INDOORS) )
    {
	if ( ch->riding != NULL )
	    do_dismount( ch, "" );
	else if ( ch->mounted_by != NULL )
	    do_dismount( ch->mounted_by, "" );
    }

    if ( ((ch->riding != NULL && !IS_AFFECTED(ch->riding, AFF_SNEAK))
    ||    (ch->riding == NULL && !IS_AFFECTED(ch, AFF_SNEAK)))
    &&   ch->invis_level < LEVEL_HERO )
    {
	act( ch->riding ? "$n rides $t." : "$n leaves $t.", 
	     ch, dir_name[door], NULL, TO_ROOM );
    }

    char_from_room( ch, FALSE );
    char_to_room( ch, to_room );
    
    if ( ch->riding != NULL
    &&   ch->riding->in_room != ch->in_room )
    {
	move_char( ch->riding, door, TRUE );
	
	if ( ch->riding->in_room != ch->in_room ) 
	{
	    ch->riding->mounted_by = NULL;
	    ch->riding = NULL; 

	    send_to_char( "You are forced to leave your mount behind.\n\r", 
	        ch );
	}
	else
	    check_improve( ch, gsn_ride, TRUE, 10, TRUE );
    }
    else if ( ch->mounted_by != NULL
    &&   ch->mounted_by->in_room != ch->in_room )
    {
	move_char( ch->mounted_by, door, TRUE );
	
	if ( ch->mounted_by->in_room != ch->in_room )
	{
	    ch->mounted_by->riding = NULL;
	    ch->mounted_by = NULL;
	    
	    send_to_char( "You are forced to leave your rider behind.\n\r", 
	        ch );
	}
	else
	    check_improve( ch->mounted_by, gsn_ride, TRUE, 10, TRUE );
    }
    
    if ( ((ch->riding != NULL && !IS_AFFECTED(ch->riding, AFF_SNEAK))
    ||    (ch->riding == NULL && !IS_AFFECTED(ch, AFF_SNEAK)))
    &&   ch->invis_level < LEVEL_HERO )
	act( "$n has arrived.", ch, NULL, NULL, TO_ROOM );

    do_look( ch, "auto" );

    if (in_room == to_room) /* no circular follows */
	return;

    for ( fch = in_room->people; fch != NULL; fch = fch_next )
    {
	fch_next = fch->next_in_room;
	
	if ( IS_AFFECTED(fch, AFF_PARALYSIS) )
	    continue;

	if ( fch->master == ch && IS_AFFECTED(fch,AFF_CHARM) 
	&&   fch->position < POS_STANDING)
	    do_stand(fch,"");

	if ( fch->master == ch && fch->position == POS_STANDING 
	&&   can_see_room(fch,to_room))
	{
	    if (IS_SET(ch->in_room->room_flags,ROOM_LAW)
	    &&  (IS_NPC(fch) && IS_SET(fch->act,ACT_AGGRESSIVE)))
	    {
		act("You can't bring $N into the city.",
		    ch,NULL,fch,TO_CHAR);
		act("You aren't allowed in the city.",
		    fch,NULL,NULL,TO_CHAR);
		continue;
	    }

	    act( "You follow $N.", fch, NULL, ch, TO_CHAR );
	    move_char( fch, door, TRUE );
	}
    }

    if ( ch->in_room->contents != NULL && !follow )
    {
	check_room_for_trap( ch, TRAP_ENTER );
	if ( CHAR_KILLED(ch) )
	    return;
    }

    /* 
     * If someone is following the char, these triggers get activated
     * for the followers before the char, but it's safer this way...
     */
    if ( IS_NPC( ch ) && HAS_TRIGGER( ch, TRIG_ENTRY ) )
	mp_percent_trigger( ch, NULL, NULL, NULL, TRIG_ENTRY );
    if ( !IS_NPC( ch ) )
    {
	mp_greet_trigger( ch );
	if ( !CHAR_KILLED(ch) )
	{
	    op_greet_trigger( ch );
	    if ( !CHAR_KILLED(ch) )
		rp_greet_trigger( ch );
	}
    }

    return;
}


void do_north( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_NORTH, FALSE );
    return;
}


void do_east( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_EAST, FALSE );
    return;
}


void do_south( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_SOUTH, FALSE );
    return;
}


void do_west( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_WEST, FALSE );
    return;
}


void do_up( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_UP, FALSE );
    return;
}


void do_down( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_DOWN, FALSE );
    return;
}


void do_northeast( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_NORTHEAST, FALSE );
    return;
}

void do_southeast( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_SOUTHEAST, FALSE );
    return;
}

void do_southwest( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_SOUTHWEST, FALSE );
    return;
}

void do_northwest( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_NORTHWEST, FALSE );
    return;
}

void do_clockwise( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_CLOCKWISE, FALSE );
    return;
}

void do_counterclockwise( CHAR_DATA *ch, char *argument )
{
    move_char( ch, DIR_COUNTERCLOCKWISE, FALSE );
    return;
}


int find_door( CHAR_DATA *ch, char *arg )
{
    EXIT_DATA *pexit;
    int door;

	 if ( !str_cmp( arg, "n" )  || !str_cmp( arg, "north" ) )    door = 0;
    else if ( !str_cmp( arg, "e" )  || !str_cmp( arg, "east"  ) )    door = 1;
    else if ( !str_cmp( arg, "s" )  || !str_cmp( arg, "south" ) )    door = 2;
    else if ( !str_cmp( arg, "w" )  || !str_cmp( arg, "west"  ) )    door = 3;
    else if ( !str_cmp( arg, "u" )  || !str_cmp( arg, "up"    ) )    door = 4;
    else if ( !str_cmp( arg, "d" )  || !str_cmp( arg, "down"  ) )    door = 5;
    else if ( !str_cmp( arg, "ne" ) || !str_cmp( arg, "northeast") ) door = 6;
    else if ( !str_cmp( arg, "se" ) || !str_cmp( arg, "southeast") ) door = 7;
    else if ( !str_cmp( arg, "sw" ) || !str_cmp( arg, "southwest") ) door = 8;
    else if ( !str_cmp( arg, "nw" ) || !str_cmp( arg, "northwest") ) door = 9;
    else if ( !str_cmp( arg, "cw" ) )                                door = 10;
    else if ( !str_cmp( arg, "ccw" ) )                               door = 11;
    else
    {
	for ( door = 0; door <= 11; door++ )
	{
	    if ( ( pexit = ch->in_room->exit[door] ) != NULL
	    &&   IS_SET(pexit->exit_info, EX_ISDOOR)
	    &&   pexit->keyword != NULL
	    &&   is_name( arg, pexit->keyword ) )
		return door;
	}
	act( "I see no $T here.", ch, NULL, arg, TO_CHAR );
	return -1;
    }

    if ( ( pexit = ch->in_room->exit[door] ) == NULL )
    {
	act( "I see no door $T here.", ch, NULL, arg, TO_CHAR );
	return -1;
    }

    if ( !IS_SET(pexit->exit_info, EX_ISDOOR) )
    {
	send_to_char( "You can't do that.\n\r", ch );
	return -1;
    }

    return door;
}



void do_open( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int door;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Open what?\n\r", ch );
	return;
    }

    if ( ( obj = get_obj_here( ch, arg ) ) != NULL )
    {
    	char *msg;
    	
 	/* open portal */
	if (obj->item_type == ITEM_PORTAL)
	{
	    if (!IS_SET(obj->value[1], EX_ISDOOR))
	    {
		send_to_char("You can't do that.\n\r",ch);
		return;
	    }

	    if (!IS_SET(obj->value[1], EX_CLOSED))
	    {
		send_to_char("It's already open.\n\r",ch);
		return;
	    }

	    if (IS_SET(obj->value[1], EX_LOCKED))
	    {
		send_to_char("It's locked.\n\r",ch);
		return;
	    }

	    REMOVE_BIT(obj->value[1], EX_CLOSED);
	    act("You open $p.",ch,obj,NULL,TO_CHAR);
	    act("$n opens $p.",ch,obj,NULL,TO_ROOM);
	    check_object_for_trap( ch, obj, TRAP_OPEN );
	    op_simple_trigger( obj, ch, OTRIG_OPEN );
	    return;
	}

	/* 'open object' */
	if ( obj->item_type != ITEM_CONTAINER)
	    { send_to_char( "That's not a container.\n\r", ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_CLOSED) )
	    { send_to_char( "It's already open.\n\r",      ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_CLOSEABLE) )
	    { send_to_char( "You can't do that.\n\r",      ch ); return; }
	if ( IS_SET(obj->value[1], CONT_LOCKED) )
	    { send_to_char( "It's locked.\n\r",            ch ); return; }

	REMOVE_BIT(obj->value[1], CONT_CLOSED);
	
	msg = find_use_msg( obj, USE_CONTAIN_OPEN_SELF );
	if ( msg != NULL )
	    act( msg, ch, obj, NULL, TO_CHAR );
	else
	    act( "You open $p.", ch, obj, NULL, TO_CHAR );
	
	msg = find_use_msg( obj, USE_CONTAIN_OPEN_ROOM );
	if ( msg != NULL )
	    act( msg, ch, obj, NULL, TO_ROOM );
	else
	    act( "$n opens $p.", ch, obj, NULL, TO_ROOM );
	    
	check_object_for_trap( ch, obj, TRAP_OPEN );
	op_simple_trigger( obj, ch, OTRIG_OPEN );
	return;
    }

    if ( ( door = find_door( ch, arg ) ) >= 0 )
    {
	/* 'open door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit = ch->in_room->exit[door];
	if ( !IS_SET(pexit->exit_info, EX_CLOSED) )
	    { send_to_char( "It's already open.\n\r",      ch ); return; }
	if (  IS_SET(pexit->exit_info, EX_LOCKED) )
	    { send_to_char( "It's locked.\n\r",            ch ); return; }

	REMOVE_BIT(pexit->exit_info, EX_CLOSED);
	act( "$n opens the $d.", ch, NULL, pexit->keyword, TO_ROOM );
	send_to_char( "Ok.\n\r", ch );

	/* open the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != NULL
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    CHAR_DATA *rch;

	    REMOVE_BIT( pexit_rev->exit_info, EX_CLOSED );
	    for ( rch = to_room->people; rch != NULL; rch = rch->next_in_room )
		act( "The $d opens.", rch, NULL, pexit_rev->keyword, TO_CHAR );

	    check_room_for_trap( ch, trap_door[door] );
	}
    }

    return;
}



void do_close( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int door;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Close what?\n\r", ch );
	return;
    }

    if ( ( obj = get_obj_here( ch, arg ) ) != NULL )
    {
	char *msg;
	
	/* portal stuff */
	if (obj->item_type == ITEM_PORTAL)
	{

	    if (!IS_SET(obj->value[1],EX_ISDOOR)
	    ||   IS_SET(obj->value[1],EX_NOCLOSE))
	    {
		send_to_char("You can't do that.\n\r",ch);
		return;
	    }

	    if (IS_SET(obj->value[1],EX_CLOSED))
	    {
		send_to_char("It's already closed.\n\r",ch);
		return;
	    }

	    SET_BIT(obj->value[1],EX_CLOSED);
	    act("You close $p.",ch,obj,NULL,TO_CHAR);
	    act("$n closes $p.",ch,obj,NULL,TO_ROOM);
	    check_object_for_trap( ch, obj, TRAP_CLOSE );
	    op_simple_trigger( obj, ch, OTRIG_CLOSE );
	    return;
	}

	/* 'close object' */
	if ( obj->item_type != ITEM_CONTAINER )
	    { send_to_char( "That's not a container.\n\r", ch ); return; }
	if ( IS_SET(obj->value[1], CONT_CLOSED) )
	    { send_to_char( "It's already closed.\n\r",    ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_CLOSEABLE) )
	    { send_to_char( "You can't do that.\n\r",      ch ); return; }

	SET_BIT(obj->value[1], CONT_CLOSED);

	msg = find_use_msg( obj, USE_CONTAIN_OPEN_SELF );
	if ( msg != NULL )
	    act( msg, ch, obj, NULL, TO_CHAR );
	else
	    act("You close $p.",ch,obj,NULL,TO_CHAR);
	
	msg = find_use_msg( obj, USE_CONTAIN_OPEN_ROOM );
	if ( msg != NULL )
	    act( msg, ch, obj, NULL, TO_ROOM );
	else
	    act( "$n closes $p.", ch, obj, NULL, TO_ROOM );

	check_object_for_trap( ch, obj, TRAP_CLOSE );
	op_simple_trigger( obj, ch, OTRIG_CLOSE );
	return;
    }

    if ( ( door = find_door( ch, arg ) ) >= 0 )
    {
	/* 'close door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit	= ch->in_room->exit[door];
	if ( IS_SET(pexit->exit_info, EX_CLOSED) )
	    { send_to_char( "It's already closed.\n\r",    ch ); return; }

	SET_BIT(pexit->exit_info, EX_CLOSED);
	act( "$n closes the $d.", ch, NULL, pexit->keyword, TO_ROOM );
	send_to_char( "Ok.\n\r", ch );

	/* close the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != 0
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    CHAR_DATA *rch;

	    SET_BIT( pexit_rev->exit_info, EX_CLOSED );
	    for ( rch = to_room->people; rch != NULL; rch = rch->next_in_room )
		act( "The $d closes.", rch, NULL, pexit_rev->keyword, TO_CHAR );

	    check_room_for_trap( ch, trap_door[door] );
	}
    }

    return;
}



bool has_key( CHAR_DATA *ch, int key )
{
    OBJ_DATA *obj;

    for ( obj = ch->carrying; obj != NULL; obj = obj->next_content )
    {
	if ( obj->pIndexData->vnum == key )
	    return TRUE;
    }

    return FALSE;
}



void do_lock( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int door;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Lock what?\n\r", ch );
	return;
    }

    if ( ( obj = get_obj_here( ch, arg ) ) != NULL )
    {
	char *msg;
	
	/* portal stuff */
	if (obj->item_type == ITEM_PORTAL)
	{
	    if (!IS_SET(obj->value[1],EX_ISDOOR)
	    ||  IS_SET(obj->value[1],EX_NOCLOSE))
	    {
		send_to_char("You can't do that.\n\r",ch);
		return;
	    }
	    if (!IS_SET(obj->value[1],EX_CLOSED))
	    {
		send_to_char("It's not closed.\n\r",ch);
	 	return;
	    }

	    if (obj->value[4] < 0 || IS_SET(obj->value[1],EX_NOLOCK))
	    {
		send_to_char("It can't be locked.\n\r",ch);
		return;
	    }

	    if (!has_key(ch,obj->value[4]))
	    {
		send_to_char("You lack the key.\n\r",ch);
		return;
	    }

	    if (IS_SET(obj->value[1],EX_LOCKED))
	    {
		send_to_char("It's already locked.\n\r",ch);
		return;
	    }

	    SET_BIT(obj->value[1],EX_LOCKED);
	    act("You lock $p.",ch,obj,NULL,TO_CHAR);
	    act("$n locks $p.",ch,obj,NULL,TO_ROOM);
	    return;
	}

	/* 'lock object' */
	if ( obj->item_type != ITEM_CONTAINER )
	    { send_to_char( "That's not a container.\n\r", ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_CLOSED) )
	    { send_to_char( "It's not closed.\n\r",        ch ); return; }
	if ( obj->value[2] < 0 )
	    { send_to_char( "It can't be locked.\n\r",     ch ); return; }
	if ( !has_key( ch, obj->value[2] ) )
	    { send_to_char( "You lack the key.\n\r",       ch ); return; }
	if ( IS_SET(obj->value[1], CONT_LOCKED) )
	    { send_to_char( "It's already locked.\n\r",    ch ); return; }

	SET_BIT(obj->value[1], CONT_LOCKED);
	
	msg = find_use_msg( obj, USE_CONTAIN_LOCK_SELF );
	if ( msg != NULL )
	    act( msg, ch, obj, NULL, TO_CHAR );
	else
	    act("You lock $p.",ch,obj,NULL,TO_CHAR);
	
	msg = find_use_msg( obj, USE_CONTAIN_LOCK_ROOM );
	if ( msg != NULL )
	    act( msg, ch, obj, NULL, TO_ROOM );
	else
	    act( "$n locks $p.", ch, obj, NULL, TO_ROOM );
	
	return;
    }

    if ( ( door = find_door( ch, arg ) ) >= 0 )
    {
	/* 'lock door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit	= ch->in_room->exit[door];
	if ( !IS_SET(pexit->exit_info, EX_CLOSED) )
	    { send_to_char( "It's not closed.\n\r",        ch ); return; }
	if ( pexit->key < 0 )
	    { send_to_char( "It can't be locked.\n\r",     ch ); return; }
	if ( !has_key( ch, pexit->key) )
	    { send_to_char( "You lack the key.\n\r",       ch ); return; }
	if ( IS_SET(pexit->exit_info, EX_LOCKED) )
	    { send_to_char( "It's already locked.\n\r",    ch ); return; }

	SET_BIT(pexit->exit_info, EX_LOCKED);
	send_to_char( "*Click*\n\r", ch );
	act( "$n locks the $d.", ch, NULL, pexit->keyword, TO_ROOM );

	/* lock the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != 0
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    SET_BIT( pexit_rev->exit_info, EX_LOCKED );
	}
    }

    return;
}



void do_unlock( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_DATA *obj;
    int door;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Unlock what?\n\r", ch );
	return;
    }

    if ( ( obj = get_obj_here( ch, arg ) ) != NULL )
    {
	char *msg;
	
 	/* portal stuff */
	if (obj->item_type == ITEM_PORTAL)
	{
	    if (!IS_SET(obj->value[1],EX_ISDOOR))
	    {
		send_to_char("You can't do that.\n\r",ch);
		return;
	    }

	    if (!IS_SET(obj->value[1],EX_CLOSED))
	    {
		send_to_char("It's not closed.\n\r",ch);
		return;
	    }

	    if (obj->value[4] < 0)
	    {
		send_to_char("It can't be unlocked.\n\r",ch);
		return;
	    }

	    if (!has_key(ch,obj->value[4]))
	    {
		send_to_char("You lack the key.\n\r",ch);
		return;
	    }

	    if (!IS_SET(obj->value[1],EX_LOCKED))
	    {
		send_to_char("It's already unlocked.\n\r",ch);
		return;
	    }

	    REMOVE_BIT(obj->value[1],EX_LOCKED);
	    act("You unlock $p.",ch,obj,NULL,TO_CHAR);
	    act("$n unlocks $p.",ch,obj,NULL,TO_ROOM);

	    check_object_for_trap( ch, obj, TRAP_UNLOCK );

	    return;
	}

	/* 'unlock object' */
	if ( obj->item_type != ITEM_CONTAINER )
	    { send_to_char( "That's not a container.\n\r", ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_CLOSED) )
	    { send_to_char( "It's not closed.\n\r",        ch ); return; }
	if ( obj->value[2] < 0 )
	    { send_to_char( "It can't be unlocked.\n\r",   ch ); return; }
	if ( !has_key( ch, obj->value[2] ) )
	    { send_to_char( "You lack the key.\n\r",       ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_LOCKED) )
	    { send_to_char( "It's already unlocked.\n\r",  ch ); return; }

	REMOVE_BIT(obj->value[1], CONT_LOCKED);
	
	msg = find_use_msg( obj, USE_CONTAIN_UNLOCK_SELF );
	if ( msg != NULL )
	    act( msg, ch, obj, NULL, TO_CHAR );
	else
	    act( "You unlock $p.",ch, obj, NULL, TO_CHAR);
	
	msg = find_use_msg( obj, USE_CONTAIN_UNLOCK_ROOM );
	if ( msg != NULL )
	    act( msg, ch, obj, NULL, TO_ROOM );
	else
	    act( "$n unlocks $p.", ch, obj, NULL, TO_ROOM );

	check_object_for_trap( ch, obj, TRAP_UNLOCK );
	return;
    }

    if ( ( door = find_door( ch, arg ) ) >= 0 )
    {
	/* 'unlock door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit = ch->in_room->exit[door];
	if ( !IS_SET(pexit->exit_info, EX_CLOSED) )
	    { send_to_char( "It's not closed.\n\r",        ch ); return; }
	if ( pexit->key < 0 )
	    { send_to_char( "It can't be unlocked.\n\r",   ch ); return; }
	if ( !has_key( ch, pexit->key) )
	    { send_to_char( "You lack the key.\n\r",       ch ); return; }
	if ( !IS_SET(pexit->exit_info, EX_LOCKED) )
	    { send_to_char( "It's already unlocked.\n\r",  ch ); return; }

	REMOVE_BIT(pexit->exit_info, EX_LOCKED);
	send_to_char( "*Click*\n\r", ch );
	act( "$n unlocks the $d.", ch, NULL, pexit->keyword, TO_ROOM );

	/* unlock the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != NULL
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    REMOVE_BIT( pexit_rev->exit_info, EX_LOCKED );
	}
    }

    return;
}



void do_pick( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *gch;
    OBJ_DATA *obj;
    int chance;
    int door;

    one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
	send_to_char( "Pick what?\n\r", ch );
	return;
    }

    if ( (chance = get_skill(ch, gsn_pick_lock)) == 0
    ||   (IS_NPC(ch) && !IS_SET(ch->act, ACT_THIEF))
    ||   (!IS_NPC(ch)
    &&    ch->level < skill_level(ch, gsn_pick_lock)) )
    {
        send_to_char("You don't know where to begin doing that!\n\r",ch);
        return;
    }

    WAIT_STATE( ch, skill_table[gsn_pick_lock].beats );

    /* look for guards */
    for ( gch = ch->in_room->people; gch; gch = gch->next_in_room )
    { 
	if ( IS_NPC(gch) 
        &&   IS_AWAKE(gch) 
        &&   effective_level(ch, gsn_pick_lock) + 5 < gch->level )
	{
	    act( "$N is standing too close to the lock.",
		ch, NULL, gch, TO_CHAR );
	    return;
	}
    }

    obj = get_obj_here( ch, arg );

    if ( obj != NULL
    &&   !IS_NPC(ch)
    &&   number_percent( ) > chance ) 
    {
	send_to_char( "You failed.\n\r", ch);
	check_improve(ch,gsn_pick_lock,FALSE,2,TRUE);

	check_object_for_trap( ch, obj, TRAP_PICK );

	return;
    }

    if ( obj != NULL )
    {
	/* portal stuff */
	if (obj->item_type == ITEM_PORTAL)
	{
	    if (!IS_SET(obj->value[1],EX_ISDOOR))
	    {
		send_to_char("You can't do that.\n\r",ch);
		return;
	    }

	    if (!IS_SET(obj->value[1],EX_CLOSED))
	    {
		send_to_char("It's not closed.\n\r",ch);
		return;
	    }

	    if (obj->value[4] < 0)
	    {
		send_to_char("It can't be unlocked.\n\r",ch);
		return;
	    }

	    if (IS_SET(obj->value[1],EX_PICKPROOF))
	    {
		send_to_char("You failed.\n\r",ch);
		return;
	    }

	    REMOVE_BIT(obj->value[1],EX_LOCKED);
	    act("You pick the lock on $p.",ch,obj,NULL,TO_CHAR);
	    act("$n picks the lock on $p.",ch,obj,NULL,TO_ROOM);
	    check_improve(ch,gsn_pick_lock,TRUE,2,TRUE);
	    
	    check_object_for_trap( ch, obj, TRAP_PICK );
	    return;
	}

	/* 'pick object' */
	if ( obj->item_type != ITEM_CONTAINER )
	    { send_to_char( "That's not a container.\n\r", ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_CLOSED) )
	    { send_to_char( "It's not closed.\n\r",        ch ); return; }
	if ( obj->value[2] < 0 )
	    { send_to_char( "It can't be unlocked.\n\r",   ch ); return; }
	if ( !IS_SET(obj->value[1], CONT_LOCKED) )
	    { send_to_char( "It's already unlocked.\n\r",  ch ); return; }
	if ( IS_SET(obj->value[1], CONT_PICKPROOF) )
	    { send_to_char( "You failed.\n\r",             ch ); return; }

	REMOVE_BIT(obj->value[1], CONT_LOCKED);
        act("You pick the lock on $p.",ch,obj,NULL,TO_CHAR);
        act("$n picks the lock on $p.",ch,obj,NULL,TO_ROOM);
	check_improve(ch,gsn_pick_lock,TRUE,2,TRUE);
	check_object_for_trap( ch, obj, TRAP_PICK );
	return;
    }

    if ( ( door = find_door( ch, arg ) ) >= 0 )
    {
	/* 'pick door' */
	ROOM_INDEX_DATA *to_room;
	EXIT_DATA *pexit;
	EXIT_DATA *pexit_rev;

	pexit = ch->in_room->exit[door];
	if ( !IS_SET(pexit->exit_info, EX_CLOSED) && !IS_IMMORTAL(ch))
	    { send_to_char( "It's not closed.\n\r",        ch ); return; }
	if ( !IS_SET(pexit->exit_info, EX_LOCKED) )
	    { send_to_char( "It's already unlocked.\n\r",  ch ); return; }
	if ( pexit->key < 0 && !IS_IMMORTAL(ch))
	    { send_to_char( "It can't be picked.\n\r",     ch ); return; }
	if ( number_percent() > chance
        ||   (IS_SET(pexit->exit_info, EX_PICKPROOF) 
	&&    !IS_IMMORTAL(ch)) )
	{
	    send_to_char( "You failed.\n\r", ch );
	    return;
	}

	REMOVE_BIT(pexit->exit_info, EX_LOCKED);
	send_to_char( "*Click*\n\r", ch );
	act( "$n picks the $d.", ch, NULL, pexit->keyword, TO_ROOM );
	check_improve(ch,gsn_pick_lock,TRUE,2,TRUE);

	/* pick the other side */
	if ( ( to_room   = pexit->u1.to_room            ) != NULL
	&&   ( pexit_rev = to_room->exit[rev_dir[door]] ) != NULL
	&&   pexit_rev->u1.to_room == ch->in_room )
	{
	    REMOVE_BIT( pexit_rev->exit_info, EX_LOCKED );
	}
    }

    return;
}




void do_stand( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj = NULL;
    char *msg;

    if ( ch->riding != NULL )
    {
	send_to_char( "Not while you are mounted.\n\r", ch );
	return;
    }

    if (argument[0] != '\0')
    {
	if (ch->position == POS_FIGHTING)
	{
	    send_to_char("Maybe you should finish fighting first?\n\r",ch);
	    return;
	}
	obj = get_obj_list(ch,argument,ch->in_room->contents);
	if (obj == NULL)
	{
	    send_to_char("You don't see that here.\n\r",ch);
	    return;
	}
	if (obj->item_type != ITEM_FURNITURE
	||  (!IS_SET(obj->value[2],STAND_AT)
	&&   !IS_SET(obj->value[2],STAND_ON)
	&&   !IS_SET(obj->value[2],STAND_IN)))
	{
	    send_to_char("You can't seem to find a place to stand.\n\r",ch);
	    return;
	}
	if (ch->on != obj && count_users(obj) >= obj->value[0])
	{
	    act_new("There's no room to stand on $p.",
		ch,obj,NULL,TO_CHAR,POS_DEAD);
	    return;
	}
 	ch->on = obj;
    }
    
    switch ( ch->position )
    {
    case POS_SLEEPING:
	if ( IS_AFFECTED(ch, AFF_SLEEP) )
	    { send_to_char( "You can't wake up!\n\r", ch ); return; }
	
	if (obj == NULL)
	{
	    send_to_char( "You wake and stand up.\n\r", ch );
	    act( "$n wakes and stands up.", ch, NULL, NULL, TO_ROOM );
	    ch->on = NULL;
	}
	else if (IS_SET(obj->value[2],STAND_AT))
	{
	    msg = find_use_msg( obj, USE_FURN_STAND_SELF );
	    if ( msg != NULL )
		act_new( msg, ch, obj, NULL, TO_CHAR, POS_DEAD );
	    else
		act_new( "You wake and stand at $p.", ch, obj, NULL, TO_CHAR, POS_DEAD );
	
	    msg = find_use_msg( obj, USE_FURN_STAND_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n wakes and stands at $p.", ch, obj, NULL, TO_ROOM );
	}
	else if (IS_SET(obj->value[2],STAND_ON))
	{
	    msg = find_use_msg( obj, USE_FURN_STAND_SELF );
	    if ( msg != NULL )
		act_new( msg, ch, obj, NULL, TO_CHAR, POS_DEAD );
	    else
		act_new( "You wake and stand on $p.", ch, obj, NULL, TO_CHAR, POS_DEAD );
	
	    msg = find_use_msg( obj, USE_FURN_STAND_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n wakes and stands on $p.", ch, obj, NULL, TO_ROOM );
	}
	else 
	{
	    msg = find_use_msg( obj, USE_FURN_STAND_SELF );
	    if ( msg != NULL )
		act_new( msg, ch, obj, NULL, TO_CHAR, POS_DEAD );
	    else
		act_new( "You wake and stand in $p.", ch, obj, NULL, TO_CHAR, POS_DEAD );
	
	    msg = find_use_msg( obj, USE_FURN_STAND_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n wakes and stands in $p.", ch, obj, NULL, TO_ROOM );
	}
	ch->position = POS_STANDING;
	do_look(ch,"auto");
	break;

    case POS_RESTING: case POS_SITTING:
	if (obj == NULL)
	{
	    send_to_char( "You stand up.\n\r", ch );
	    act( "$n stands up.", ch, NULL, NULL, TO_ROOM );
	    ch->on = NULL;
	}
	else if (IS_SET(obj->value[2],STAND_AT))
	{
	    msg = find_use_msg( obj, USE_FURN_STAND_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You stand at $p.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_STAND_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n stands at $p.", ch, obj, NULL, TO_ROOM );
	}
	else if (IS_SET(obj->value[2],STAND_ON))
	{
	    msg = find_use_msg( obj, USE_FURN_STAND_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You stand on $p.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_STAND_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n stands on $p.", ch, obj, NULL, TO_ROOM );
	}
	else
	{
	    msg = find_use_msg( obj, USE_FURN_STAND_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You stand in $p.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_STAND_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n stands in $p.", ch, obj, NULL, TO_ROOM );
	}
	ch->position = POS_STANDING;
	break;

    case POS_STANDING:
	send_to_char( "You are already standing.\n\r", ch );
	break;

    case POS_FIGHTING:
	send_to_char( "You are already fighting!\n\r", ch );
	break;
    }

    return;
}



void do_rest( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj = NULL;
    char *msg;

    if (ch->position == POS_FIGHTING)
    {
	send_to_char("You are already fighting!\n\r",ch);
	return;
    }
    
    if ( ch->riding != NULL )
    {
	send_to_char( "Not while you are mounted.\n\r", ch );
	return;
    }

    /* okay, now that we know we can rest, find an object to rest on */
    if (argument[0] != '\0')
    {
	obj = get_obj_list(ch,argument,ch->in_room->contents);
	if (obj == NULL)
	{
	    send_to_char("You don't see that here.\n\r",ch);
	    return;
	}
    }
    else obj = ch->on;

    if (obj != NULL)
    {
        if (!IS_SET(obj->item_type,ITEM_FURNITURE) 
    	||  (!IS_SET(obj->value[2],REST_ON)
    	&&   !IS_SET(obj->value[2],REST_IN)
    	&&   !IS_SET(obj->value[2],REST_AT)))
    	{
	    send_to_char("You can't rest on that.\n\r",ch);
	    return;
    	}

        if (obj != NULL && ch->on != obj && count_users(obj) >= obj->value[0])
        {
	    act_new("There's no more room on $p.",ch,obj,NULL,TO_CHAR,POS_DEAD);
	    return;
    	}
	
	ch->on = obj;
    }

    switch ( ch->position )
    {
    case POS_SLEEPING:
	if (IS_AFFECTED(ch,AFF_SLEEP))
	{
	    send_to_char("You can't wake up!\n\r",ch);
	    return;
	}

	if (obj == NULL)
	{
	    send_to_char( "You wake up and start resting.\n\r", ch );
	    act ("$n wakes up and starts resting.",ch,NULL,NULL,TO_ROOM);
	}
	else if (IS_SET(obj->value[2],REST_AT))
	{
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act_new( msg, ch, obj, NULL, TO_CHAR, POS_SLEEPING );
	    else
		act_new( "You wake up and rest at $p.",
		    ch, obj, NULL, TO_CHAR, POS_SLEEPING );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n wakes up and rests at $p.", ch, obj, NULL, TO_ROOM );
	}
        else if (IS_SET(obj->value[2],REST_ON))
        {
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act_new( msg, ch, obj, NULL, TO_CHAR, POS_SLEEPING );
	    else
		act_new( "You wake up and rest on $p.",
                    ch, obj, NULL, TO_CHAR, POS_SLEEPING );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n wakes up and rests on $p.", ch, obj, NULL, TO_ROOM );
        }
        else
        {
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act_new( msg, ch, obj, NULL, TO_CHAR, POS_SLEEPING );
	    else
		act_new( "You wake up and rest in $p.",
                    ch, obj, NULL, TO_CHAR, POS_SLEEPING );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n wakes up and rests in $p.", ch, obj, NULL, TO_ROOM );
        }
	ch->position = POS_RESTING;
	break;

    case POS_RESTING:
	send_to_char( "You are already resting.\n\r", ch );
	break;

    case POS_STANDING:
	if (obj == NULL)
	{
	    send_to_char( "You rest.\n\r", ch );
	    act( "$n sits down and rests.", ch, NULL, NULL, TO_ROOM );
	}
        else if (IS_SET(obj->value[2],REST_AT))
        {
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You sit down at $p and rest.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n sits down at $p and rests.", ch, obj, NULL, TO_ROOM );
        }
        else if (IS_SET(obj->value[2],REST_ON))
        {
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You sit down on $p and rest.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n sits down on $p and rests.", ch, obj, NULL, TO_ROOM );
        }
        else
        {
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You rest in $p.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n rests in $p.", ch, obj, NULL, TO_ROOM );
        }
	ch->position = POS_RESTING;
	break;

    case POS_SITTING:
	if (obj == NULL)
	{
	    send_to_char("You rest.\n\r",ch);
	    act("$n rests.",ch,NULL,NULL,TO_ROOM);
	}
        else if (IS_SET(obj->value[2],REST_AT))
        {
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You rest at $p.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n rests at $p.", ch, obj, NULL, TO_ROOM );
        }
        else if (IS_SET(obj->value[2],REST_ON))
        {
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You rest on $p.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n rests on $p.", ch, obj, NULL, TO_ROOM );
        }
        else
        {
	    msg = find_use_msg( obj, USE_FURN_REST_SELF );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_CHAR );
	    else
		act( "You rest in $p.", ch, obj, NULL, TO_CHAR );
	
	    msg = find_use_msg( obj, USE_FURN_REST_ROOM );
	    if ( msg != NULL )
		act( msg, ch, obj, NULL, TO_ROOM );
	    else
		act( "$n rests in $p.", ch, obj, NULL, TO_ROOM );
	}
	ch->position = POS_RESTING;
	break;
    }


    return;
}


void do_sit (CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj = NULL;
    char *msg;

    if (ch->position == POS_FIGHTING)
    {
	send_to_char("Maybe you should finish this fight first?\n\r",ch);
	return;
    }

    if ( ch->riding != NULL )
    {
	send_to_char( "Not while you are mounted.\n\r", ch );
	return;
    }

    /* okay, now that we know we can sit, find an object to sit on */
    if (argument[0] != '\0')
    {
	obj = get_obj_list(ch,argument,ch->in_room->contents);
	if (obj == NULL)
	{
	    send_to_char("You don't see that here.\n\r",ch);
	    return;
	}
    }
    else obj = ch->on;

    if (obj != NULL)                                                              
    {
	if (!IS_SET(obj->item_type,ITEM_FURNITURE)
	||  (!IS_SET(obj->value[2],SIT_ON)
	&&   !IS_SET(obj->value[2],SIT_IN)
	&&   !IS_SET(obj->value[2],SIT_AT)))
	{
	    send_to_char("You can't sit on that.\n\r",ch);
	    return;
	}

	if (obj != NULL && ch->on != obj && count_users(obj) >= obj->value[0])
	{
	    act_new("There's no more room on $p.",ch,obj,NULL,TO_CHAR,POS_DEAD);
	    return;
	}

	ch->on = obj;
    }
    switch (ch->position)
    {
	case POS_SLEEPING:
	    if (IS_AFFECTED(ch,AFF_SLEEP))
	    {
		send_to_char("You can't wake up!\n\r",ch);
		return;
	    }

            if (obj == NULL)
            {
            	send_to_char( "You wake and sit up.\n\r", ch );
            	act( "$n wakes and sits up.", ch, NULL, NULL, TO_ROOM );
            }
            else if (IS_SET(obj->value[2],SIT_AT))
            {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act_new( msg, ch, obj, NULL, TO_CHAR, POS_DEAD );
		else
		    act_new( "You wake and sit at $p.",
		    	ch, obj, NULL, TO_CHAR, POS_DEAD );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n wakes and sits at $p.", ch, obj, NULL, TO_ROOM );
            }
            else if (IS_SET(obj->value[2],SIT_ON))
            {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act_new( msg, ch, obj, NULL, TO_CHAR, POS_DEAD );
		else
		    act_new( "You wake and sit on $p.",
		    	ch, obj, NULL, TO_CHAR, POS_DEAD );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n wakes and sits on $p.", ch, obj, NULL, TO_ROOM );
            }
            else
            {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act_new( msg, ch, obj, NULL, TO_CHAR, POS_DEAD );
		else
		    act_new( "You wake and sit in $p.",
		    	ch, obj, NULL, TO_CHAR, POS_DEAD );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n wakes and sits in $p.", ch, obj, NULL, TO_ROOM );
            }

	    ch->position = POS_SITTING;
	    break;
	case POS_RESTING:
	    if (obj == NULL)
		send_to_char("You stop resting.\n\r",ch);
	    else if (IS_SET(obj->value[2],SIT_AT))
	    {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act( "You sit at $p.",
		    	ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n sits at $p.", ch, obj, NULL, TO_ROOM );
	    }

	    else if (IS_SET(obj->value[2],SIT_ON))
	    {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act( "You sit on $p.",
		    	ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n sits on $p.", ch, obj, NULL, TO_ROOM );
	    }
	    else
	    {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act( "You sit in $p.",
		    	ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n sits in $p.", ch, obj, NULL, TO_ROOM );
	    }
	    ch->position = POS_SITTING;
	    break;
	case POS_SITTING:
	    send_to_char("You are already sitting down.\n\r",ch);
	    break;
	case POS_STANDING:
	    if (obj == NULL)
    	    {
		send_to_char("You sit down.\n\r",ch);
    	        act("$n sits down on the ground.",ch,NULL,NULL,TO_ROOM);
	    }
	    else if (IS_SET(obj->value[2],SIT_AT))
	    {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act( "You sit down at $p.",
		    	ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n sits down at $p.", ch, obj, NULL, TO_ROOM );
	    }
	    else if (IS_SET(obj->value[2],SIT_ON))
	    {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act( "You sit on $p.",
		    	ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n sits on $p.", ch, obj, NULL, TO_ROOM );
	    }
	    else
	    {
		msg = find_use_msg( obj, USE_FURN_SIT_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act( "You sit in $p.",
		    	ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SIT_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n sits in $p.", ch, obj, NULL, TO_ROOM );
	    }
    	    ch->position = POS_SITTING;
    	    break;
    }
    return;
}


void do_sleep( CHAR_DATA *ch, char *argument )
{
    OBJ_DATA *obj = NULL;

    if ( ch->riding != NULL )
    {
	send_to_char( "Not while you are mounted.\n\r", ch );
	return;
    }

    switch ( ch->position )
    {
    case POS_SLEEPING:
	send_to_char( "You are already sleeping.\n\r", ch );
	break;

    case POS_RESTING:
    case POS_SITTING:
    case POS_STANDING: 
	if (argument[0] == '\0' && ch->on == NULL)
	{
	    send_to_char( "You go to sleep.\n\r", ch );
	    act( "$n goes to sleep.", ch, NULL, NULL, TO_ROOM );
	    ch->position = POS_SLEEPING;
	}
	else  /* find an object and sleep on it */
	{
	    char *msg;
	    
	    if (argument[0] == '\0')
		obj = ch->on;
	    else
	    	obj = get_obj_list( ch, argument,  ch->in_room->contents );

	    if (obj == NULL)
	    {
		send_to_char("You don't see that here.\n\r",ch);
		return;
	    }
	    if (obj->item_type != ITEM_FURNITURE
	    ||  (!IS_SET(obj->value[2],SLEEP_ON) 
	    &&   !IS_SET(obj->value[2],SLEEP_IN)
	    &&	 !IS_SET(obj->value[2],SLEEP_AT)))
	    {
		send_to_char("You can't sleep on that!\n\r",ch);
		return;
	    }

	    if (ch->on != obj && count_users(obj) >= obj->value[0])
	    {
		act_new("There is no room on $p for you.",
		    ch,obj,NULL,TO_CHAR,POS_DEAD);
		return;
	    }

	    ch->on = obj;
	    if (IS_SET(obj->value[2],SLEEP_AT))
	    {
		msg = find_use_msg( obj, USE_FURN_SLEEP_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act("You go to sleep at $p.", ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SLEEP_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n goes to sleep at $p.", ch, obj, NULL, TO_ROOM );
	    }
	    else if (IS_SET(obj->value[2],SLEEP_ON))
	    {
		msg = find_use_msg( obj, USE_FURN_SLEEP_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act("You go to sleep on $p.", ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SLEEP_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n goes to sleep on $p.", ch, obj, NULL, TO_ROOM );
	    }
	    else
	    {
		msg = find_use_msg( obj, USE_FURN_SLEEP_SELF );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_CHAR );
		else
		    act("You go to sleep in $p.", ch, obj, NULL, TO_CHAR );
	
		msg = find_use_msg( obj, USE_FURN_SLEEP_ROOM );
		if ( msg != NULL )
		    act( msg, ch, obj, NULL, TO_ROOM );
		else
		    act( "$n goes to sleep in $p.", ch, obj, NULL, TO_ROOM );
	    }
	    ch->position = POS_SLEEPING;
	}
	break;

    case POS_FIGHTING:
	send_to_char( "You are already fighting!\n\r", ch );
	break;
    }

    return;
}



void do_wake( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	if ( ch->riding == NULL )
	    do_stand( ch, argument );
	else
	{
	    send_to_char( 
	        "You awake from your slumber... still mounted!\n\r",
	        ch );
	    do_look( ch, "auto" );
	}
	return;
    }

    if ( !IS_AWAKE(ch) )
	{ send_to_char( "You are asleep yourself!\n\r",       ch ); return; }

    if ( ( victim = get_char_room( ch, arg ) ) == NULL )
	{ send_to_char( "They aren't here.\n\r",              ch ); return; }

    if ( IS_AWAKE(victim) )
	{ act( "$N is already awake.", ch, NULL, victim, TO_CHAR ); return; }

    if ( IS_AFFECTED(victim, AFF_SLEEP) )
	{ act( "You can't wake $M!",   ch, NULL, victim, TO_CHAR );  return; }

    act_new( "$n wakes you.", ch, NULL, victim, TO_VICT,POS_SLEEPING );
    do_stand(victim,"");
    return;
}



void do_sneak( CHAR_DATA *ch, char *argument )
{
    AFFECT_DATA af;

    send_to_char( "You attempt to move silently.\n\r", ch );
    affect_strip( ch, gsn_sneak );

    if (IS_AFFECTED(ch,AFF_SNEAK))
	return;

    if ( number_percent( ) < get_skill(ch,gsn_sneak))
    {
	check_improve(ch,gsn_sneak,TRUE,3,TRUE);
	af.where	= TO_AFFECTS;
	af.type		= gsn_sneak;
	af.level	= ch->level; 
	af.duration	= ch->level;
	af.location	= APPLY_NONE;
	af.modifier	= 0;
	af.bitvector	= AFF_SNEAK;
	af.procNo	= 0;
	af.flags	= 0;
	af.name		= "";
	af.appearance	= "";
	affect_to_char( ch, &af );
    }
    else
	check_improve(ch,gsn_sneak,FALSE,3,TRUE);

    return;
}



void do_hide( CHAR_DATA *ch, char *argument )
{
    send_to_char( "You attempt to hide.\n\r", ch );

    if ( IS_AFFECTED(ch, AFF_HIDE) )
	REMOVE_BIT(ch->affected_by, AFF_HIDE);

    if ( ch->riding != NULL && !IS_AFFECTED(ch->riding, AFF_HIDE) )
    {
	send_to_char( "You can't do that while mounted!\n\r", ch );
	return;
    }

    if ( number_percent( ) < get_skill(ch,gsn_hide))
    {
	SET_BIT(ch->affected_by, AFF_HIDE);
	check_improve(ch,gsn_hide,TRUE,3,TRUE);
    }
    else
	check_improve(ch,gsn_hide,FALSE,3,TRUE);

    return;
}



/*
 * Contributed by Alander.
 */
void do_visible( CHAR_DATA *ch, char *argument )
{
    affect_strip ( ch, gsn_invis			);
    affect_strip ( ch, gsn_mass_invis			);
    affect_strip ( ch, gsn_improved_invis		);
    affect_strip ( ch, gsn_sneak			);
    affect_strip ( ch, gsn_shadow			);
    REMOVE_BIT   ( ch->affected_by, AFF_HIDE		);
    REMOVE_BIT   ( ch->affected_by, AFF_INVISIBLE	);
    REMOVE_BIT   ( ch->affected_by2,AFF2_IMPROVED_INVIS	);
    REMOVE_BIT   ( ch->affected_by, AFF_SNEAK		);
    send_to_char( "Ok.\n\r", ch );
    return;
}



void do_recall( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    ROOM_INDEX_DATA *location;
    CHAR_DATA *victim;
    int god;

    if ( IS_NPC(ch) && !IS_SET(ch->act, ACT_PET) && ch->mounted_by == NULL )
    {
	send_to_char( "Only players can recall.\n\r", ch );
	return;
    }

    if ( IS_AFFECTED(ch, AFF_CHARM)
    &&   ch->master != NULL
    &&   ch->master->in_room == ch->in_room )
    {
	send_to_char( "What?  And leave your beloved master?\n\r", ch );
	return;
    }
    
    if ( !IS_NPC(ch) && ch->pcdata->heresy >= HERESY_NORECALL )
    {
        act( "$t refuses to answer your prayers.",
             ch, religion_table[GET_GOD(ch)].head, NULL, TO_CHAR );
        return;
    }
  
    act( "$n prays for transportation!", ch, 0, 0, TO_ROOM );

    if ( IS_NPC(ch) && ch->master != NULL )
	god = GET_GOD(ch->master);
    else
	god = GET_GOD(ch);

    if ( religion_table[god].recall_vnum )
	location = get_room_index( religion_table[god].recall_vnum );
    else
	location = get_room_index( ROOM_VNUM_TEMPLE );

    if ( location == NULL )
    {
	send_to_char( "You are completely lost.\n\r", ch );
	return;
    }

    if ( ch->in_room == location )
	return;

    if ( IS_SET(ch->in_room->room_flags, ROOM_NO_RECALL)
    ||   IS_AFFECTED(ch, AFF_CURSE))
    {
	act( "$t has forsaken you.",
	    ch, religion_table[GET_GOD(ch)].head, NULL, TO_CHAR );
	return;
    }

    if ( ( victim = ch->fighting ) != NULL )
    {
	int lose,skill;
	
	skill = get_skill(ch,gsn_recall);

	if ( number_percent() < 80 * skill / 100 )
	{
	    check_improve(ch,gsn_recall,FALSE,6,TRUE);
	    WAIT_STATE( ch, 4 );
	    sprintf( buf, "You failed!.\n\r");
	    send_to_char( buf, ch );
	    return;
	}


	lose = (ch->desc != NULL) ? 25 : 50;
	gain_exp( ch, 0 - lose );
	sprintf( buf, "You recall from combat!  You lose %d exps.\n\r", lose );
	send_to_char( buf, ch );
	
	adjust_flee_renown( ch );
	
	check_improve( ch, gsn_consider, TRUE, 1, FALSE );
	check_improve(ch,gsn_recall,TRUE,4,TRUE);
	
	save_hostility( ch );
	stop_fighting( ch, TRUE );
    }

    ch->move = ch->move * 3 / 4;

    if ( !IS_NULLSTR(religion_table[god].recall_self) )
	act( religion_table[god].recall_self, ch, NULL, NULL, TO_CHAR );
    if ( !IS_NULLSTR(religion_table[god].recall_room) )
	act( religion_table[god].recall_room, ch, NULL, NULL, TO_ROOM );
    else
	act( "$n disappears.", ch, NULL, NULL, TO_ROOM );
    char_from_room( ch, FALSE );
    char_to_room( ch, location );
    if ( !IS_NULLSTR(religion_table[god].recall_appear_self) )
	act( religion_table[god].recall_appear_self, ch, NULL, NULL, TO_CHAR );
    if ( !IS_NULLSTR(religion_table[god].recall_appear_room) )
	act( religion_table[god].recall_appear_room, ch, NULL, NULL, TO_ROOM );
    else
	act( "$n appears in the room.", ch, NULL, NULL, TO_ROOM );
    do_look( ch, "auto" );

/*
 *  if ( ch->mounted_by != NULL )
 *  {
 *	if ( ch->mounted_by->in_room != ch->in_room )
 *	{
 *	    do_recall( ch->mounted_by, "" );
 *
 *	    if ( ch->mounted_by->in_room != ch->in_room )
 *	    {
 *		ch->mounted_by->riding = NULL;
 *		ch->mounted_by = NULL;
 *	    }
 *	}
 *  }
 *  else
 */
    if ( ch->riding != NULL )
    {
	if ( ch->riding->in_room != ch->in_room )
	{
	    do_recall( ch->riding, "" );

	    if ( ch->riding->in_room != ch->in_room )
	    {
		ch->riding->mounted_by = NULL;
		ch->riding = NULL;
	    }
	}
    }
    else if ( ch->pet != NULL )
	do_recall( ch->pet, "" );

    /* 
     * If someone is following the char, these triggers get activated
     * for the followers before the char, but it's safer this way...
     */
    if ( IS_NPC(ch) && HAS_TRIGGER(ch, TRIG_ENTRY) )
	mp_percent_trigger( ch, NULL, NULL, NULL, TRIG_ENTRY );
    if ( !IS_NPC(ch) )
	mp_greet_trigger( ch );

    return;
}



void do_train( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *mob;
    sh_int stat = - 1;
    char *pOutput = NULL;
    int cost;

    if ( IS_NPC(ch) )
	return;

    /*
     * Check for trainer.
     */
    for ( mob = ch->in_room->people; mob; mob = mob->next_in_room )
    {
	if ( IS_NPC(mob) && IS_SET(mob->act, ACT_TRAIN) )
	    break;
    }

    if ( mob == NULL )
    {
	send_to_char( "You can't do that here.\n\r", ch );
	return;
    }

    if ( argument[0] == '\0' )
    {
	sprintf( buf, "You have %d training sessions.\n\r", ch->train );
	send_to_char( buf, ch );
	argument = "foo";
    }

/*    
 *    if ( ch->pcdata->true_level == LEVEL_HERO )
 *    {
 *	send_to_char( "You have reached the limit of your abilities.\n\r", ch );
 *	return;
 *    }
 */

    cost = 1;

    if ( !str_cmp( argument, "str" ) )
    {
	if ( class_table[ch->class].attr_prime == STAT_STR )
	    cost    = 1;
	stat        = STAT_STR;
	pOutput     = "strength";
    }

    else if ( !str_cmp( argument, "int" ) )
    {
	if ( class_table[ch->class].attr_prime == STAT_INT )
	    cost    = 1;
	stat	    = STAT_INT;
	pOutput     = "intelligence";
    }

    else if ( !str_cmp( argument, "wis" ) )
    {
	if ( class_table[ch->class].attr_prime == STAT_WIS )
	    cost    = 1;
	stat	    = STAT_WIS;
	pOutput     = "wisdom";
    }

    else if ( !str_cmp( argument, "dex" ) )
    {
	if ( class_table[ch->class].attr_prime == STAT_DEX )
	    cost    = 1;
	stat  	    = STAT_DEX;
	pOutput     = "dexterity";
    }

    else if ( !str_cmp( argument, "con" ) )
    {
	if ( class_table[ch->class].attr_prime == STAT_CON )
	    cost    = 1;
	stat	    = STAT_CON;
	pOutput     = "constitution";
    }

    else if ( !str_cmp( argument, "per" ) )
    {
	if ( class_table[ch->class].attr_prime == STAT_PER )
	    cost    = 1;
	stat	    = STAT_PER;
	pOutput     = "perception";
    }
    
    else if ( !str_cmp( argument, "cha" ) )
    {
	if ( class_table[ch->class].attr_prime == STAT_CHA )
	    cost    = 1;
	stat	    = STAT_CHA;
	pOutput     = "charisma";
    }

    else if ( !str_cmp(argument, "hp" ) )
	cost = 1;

    else if ( !str_cmp(argument, "mana" ) )
	cost = 1;

    else
    {
	strcpy( buf, "You can train:" );
	if ( ch->perm_stat[STAT_STR] < get_max_train(ch,STAT_STR)) 
	    strcat( buf, " str" );
	if ( ch->perm_stat[STAT_INT] < get_max_train(ch,STAT_INT))  
	    strcat( buf, " int" );
	if ( ch->perm_stat[STAT_WIS] < get_max_train(ch,STAT_WIS)) 
	    strcat( buf, " wis" );
	if ( ch->perm_stat[STAT_DEX] < get_max_train(ch,STAT_DEX))  
	    strcat( buf, " dex" );
	if ( ch->perm_stat[STAT_CON] < get_max_train(ch,STAT_CON))  
	    strcat( buf, " con" );
	if ( ch->perm_stat[STAT_PER] < get_max_train(ch,STAT_PER))
	    strcat( buf, " per" );
	if ( ch->perm_stat[STAT_CHA] < get_max_train(ch,STAT_CHA))
	    strcat( buf, " cha" );
	strcat( buf, " hp mana");

	if ( buf[strlen(buf)-1] != ':' )
	{
	    strcat( buf, ".\n\r" );
	    send_to_char( buf, ch );
	}
	else
	{
	    /*
	     * This message dedicated to Jordan ... you big stud!
	     */
	    act( "You have nothing left to train, you $T!",
		ch, NULL,
		ch->sex == SEX_MALE   ? "big stud" :
		ch->sex == SEX_FEMALE ? "hot babe" :
					"wild thing",
		TO_CHAR );
	}

	return;
    }

    if (!str_cmp("hp",argument))
    {
    	if ( cost > ch->train )
    	{
       	    send_to_char( "You don't have enough training sessions.\n\r", ch );
            return;
        }
 
	ch->train -= cost;
        ch->pcdata->perm_hit += 10;
        ch->max_hit += 10;
        ch->hit +=10;
        act( "Your durability increases!",ch,NULL,NULL,TO_CHAR);
        act( "$n's durability increases!",ch,NULL,NULL,TO_ROOM);
        return;
    }
 
    if (!str_cmp("mana",argument))
    {
        if ( cost > ch->train )
        {
            send_to_char( "You don't have enough training sessions.\n\r", ch );
            return;
        }

	ch->train -= cost;
        ch->pcdata->perm_mana += 10;
        ch->max_mana += 10;
        ch->mana += 10;
        act( "Your power increases!",ch,NULL,NULL,TO_CHAR);
        act( "$n's power increases!",ch,NULL,NULL,TO_ROOM);
        return;
    }

    if ( ch->perm_stat[stat]  >= get_max_train(ch,stat) )
    {
	act( "Your $T is already at maximum.", ch, NULL, pOutput, TO_CHAR );
	return;
    }

    if ( cost > ch->train )
    {
	send_to_char( "You don't have enough training sessions.\n\r", ch );
	return;
    }

    ch->train		-= cost;
  
    ch->perm_stat[stat] = UMIN( ch->perm_stat[stat] + 4, 
                                get_max_train(ch,stat) );
    act( "Your $T increases!", ch, NULL, pOutput, TO_CHAR );
    act( "$n's $T increases!", ch, NULL, pOutput, TO_ROOM );
    return;
}


void do_petition( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    int clan;

    if ( IS_NPC(ch) )
    {
	send_to_char( "Only players may join a clan.\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg);

    if ( arg[0] == '\0' )
    {
        send_to_char( "Syntax: petition <clan>\n\r",ch);
        return;
    }

    if (clan_lookup(arg) == ch->clan)
    {
	send_to_char( "You already belong to that clan!\n\r",ch);
	return;
    }

    if (!str_prefix(arg,"none"))
    {
        send_to_char("You now apply to no clan.\n\r",ch);
        ch->pcdata->petition =  0;
        return;
    }

    if ((clan = clan_lookup(arg)) == 0)
    {
        send_to_char("No such clan exists.\n\r",ch);
        return;
    }

    if (clan_table[clan].independent)
    {
	send_to_char("You must ask an immortal for that.\n\r",ch);
	return;
    }

    sprintf(buf,"You have petitioned clan %s for admission.\n\r",
	capitalize(clan_table[clan].name));
    send_to_char(buf,ch);
    ch->pcdata->petition = clan_lookup(arg);

   return;
}


void do_accept( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    argument = one_argument( argument, arg1 );

    if (IS_NPC(ch) || !ch->pcdata->leader)
    {
	send_to_char( "You are not empowered to do that.\n\r",ch);
	return;
    }

    if ( arg1[0] == '\0' )
    {
	send_to_char( "Syntax: accept <char>\n\r",ch);
	return;
    }

    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
	send_to_char( "They aren't playing.\n\r", ch );
	return;
    }
    if ( IS_NPC(victim) )
    {
	send_to_char( "You can only accept players into your clan.\n\r", ch );
	return;
    }

    if (victim->pcdata->petition != ch->clan)
    {
	send_to_char("They have not petitioned your clan.\n\r",ch);
	return;
    }

    if (victim == ch)
    {
	send_to_char( "It's good to like yourself that much.\n\r",ch);
	return;
    }

    victim->clan = ch->clan;
    send_to_char( "You have accepted a new clansmember.\n\r",ch);
    send_to_char( "You have been accepted as a clansmember!\n\r,",victim);
    ch->pcdata->petition = 0;
    return;
}

void do_defrock( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH], arg2[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( IS_NPC(ch) || ch->pcdata->leader < 5 )
    {
	send_to_char("You are not empowered to do that.\n\r",ch);
	return;
    }

    if ( arg1[0] == '\0' )
    {
        send_to_char( "Syntax: defrock <char> [deputy]\n\r",ch);
        return;
    }

    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
	send_to_char( "They aren't playing.\n\r", ch );
	return;
    }
    if ( IS_NPC(victim) )
    {
	send_to_char( "You can only defrock players.\n\r", ch );
	return;
    }

    if (victim->clan != ch->clan)
    {
	send_to_char( "They aren't a member of your clan.\n\r", ch );
	return;
    }

    if (victim == ch)
    {
	send_to_char("Such judgement is only for the gods above.\n\r",ch);
	return;
    }

    if ( arg2[0] != '\0' && !str_prefix(arg2, "deputy") )
    {
	send_to_char("They are no longer your deputy.\n\r",ch);
	send_to_char("You are no longer deputized!\n\r",victim);
	victim->pcdata->leader = 0;
	return;
    }

    victim->clan = 0;
    send_to_char("They are now clanless.\n\r",ch);
    send_to_char("You are now a member of no clan!\n\r",victim);
    victim->clan = 0;
    return;
}


void do_deputize( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    CHAR_DATA *victim;

    argument = one_argument( argument, arg1 );

    if ( IS_NPC(ch) || ch->pcdata->leader < 5 )
    {
	send_to_char( "You are not empowered to do that.\n\r",ch);
	return;
    }

    if ( arg1[0] == '\0' )
    {
	send_to_char( "Syntax: deputize <char>\n\r",ch);
	return;
    }

    if ( ( victim = get_char_world( ch, arg1 ) ) == NULL )
    {
	send_to_char( "They aren't playing.\n\r", ch );
	return;
    }
    if ( IS_NPC(victim) )
    {
	send_to_char( "You can only deputize players.\n\r", ch );
	return;
    }

    if (victim == ch)
    {
	send_to_char( "You're your own best friend?\n\r",ch);
	return;
    }

    if (victim->clan != ch->clan)
    {
	send_to_char( "That person is not a member of your clan.\n\r",ch);
	return;
    }

    victim->pcdata->leader = 1;
    send_to_char( "You have accepted a new deputy.\n\r",ch);
    send_to_char( "You have been deputized!\n\r",victim);
    return;
}


void do_escape( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    int chance;
    int door;

    if ( ch->fighting == NULL )
    {
	send_to_char( "But you're not fighting anyone!?\n\r", ch );
	return;
    }
    
    if ( ch->riding != NULL )
    {
	send_to_char( "Not while you're riding.\n\r", ch );
	return;
    }
    
    if ( (chance = get_skill(ch, gsn_escape)) == 0 )
    {
	send_to_char( "You huddle up in a fetal ball and try the escape the world.\n\r", ch );
	return;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] != '\0' )
    {
	EXIT_DATA *pexit;
	
	     if ( !str_cmp( arg, "n"   ) || !str_cmp( arg, "north"		) ) door = 0;
	else if ( !str_cmp( arg, "e"   ) || !str_cmp( arg, "east"		) ) door = 1;
	else if ( !str_cmp( arg, "s"   ) || !str_cmp( arg, "south"		) ) door = 2;
	else if ( !str_cmp( arg, "w"   ) || !str_cmp( arg, "west"		) ) door = 3;
	else if ( !str_cmp( arg, "u"   ) || !str_cmp( arg, "up"			) ) door = 4;
	else if ( !str_cmp( arg, "d"   ) || !str_cmp( arg, "down"		) ) door = 5;
	else if ( !str_cmp( arg, "ne"  ) || !str_cmp ( arg, "northeast"		) ) door = 6;
	else if ( !str_cmp( arg, "se"  ) || !str_cmp ( arg, "southeast"		) ) door = 7;
	else if ( !str_cmp( arg, "sw"  ) || !str_cmp ( arg, "southwest"		) ) door = 8;
	else if ( !str_cmp( arg, "nw"  ) || !str_cmp ( arg, "northwest"		) ) door = 9;
	else if ( !str_cmp( arg, "cw"  ) || !str_cmp ( arg, "clockwise"		) ) door = 10;
	else if ( !str_cmp( arg, "ccw" ) || !str_cmp ( arg, "counterclockwise"	) ) door = 11;
	else
	{
	    send_to_char( "Escape in which direction?\n\r", ch );
	    return;
	}
	
	if ( (pexit = ch->in_room->exit[door]) == 0
	||   pexit->u1.to_room == NULL )
	{
	    send_to_char( "There doesn't seem to be an exit in that direction.\n\r", ch );
	    return;
	}
	
	if ( IS_SET(pexit->exit_info, EX_CLOSED) )
	{
	    send_to_char( "Better try opening the door first.\n\r", ch );
	    return;
	}

	if ( IS_NPC(ch)
	&&   IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB) )
	{
	    send_to_char( "You're not allowed in there.", ch );
	    return;
	}
	
	chance /= 2;
    }
    else
    {
	EXIT_DATA *pexit;
	int i;
	
	for ( i=0; i<6; i++ )
	{
	    door = number_door();
	
	    if ( (pexit = ch->in_room->exit[door]) == 0
	    ||   pexit->u1.to_room == NULL
	    ||   IS_SET(pexit->exit_info, EX_CLOSED)
	    ||   (IS_NPC(ch)
	    &&    IS_SET(pexit->u1.to_room->room_flags, ROOM_NO_MOB) ) )
		continue;
	
	    break;
	}
	
	if ( i == 6 )
	{
	    send_to_char( "You can't seem to find anywhere to run too!\n\r", ch );
	   
	    chance = 0;
	}
    }
 
    if ( number_percent() > chance )
    {
	send_to_char( "PANIC! You failed to escape the fight!\n\r", ch );
	
	WAIT_STATE( ch, skill_table[gsn_escape].beats );
	check_improve( ch, gsn_escape, FALSE, 1, TRUE );
    }
    else
    {
        CHAR_DATA *opponent = ch->fighting;

	save_hostility( ch );
	act( "$n has fled!", ch, NULL, NULL, TO_ROOM );

	move_char( ch, door, FALSE );
    
	
	if ( ch->in_room != opponent->in_room )
        {
            send_to_char( "You sneak away safely.\n\r", ch );
            stop_fighting( ch, TRUE );
        }
	
	WAIT_STATE( ch, skill_table[gsn_escape].beats );
	check_improve( ch, gsn_escape, TRUE, 1, TRUE );
    }
}
