/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "merc.h"
#include "db.h"
#include "olc.h"
#include "tables.h"
#include "recycle.h"

/*
 * Type definitions.
 */
struct create_type {
    char *	name;
    int		olc_level;
};


/*
 * Local constants.
 */
static struct create_type create_levels[] = {
    { "object",			1 },
    { "mobile",			1 },
    { "room",			1 },
    { "program",		4 },
    { NULL }
};


/*
 * Global function prototypes.
 */
int     flag_lookup     args( ( const char *name,
                                const struct flag_type *flag_table ) );


/*
 * Local function prototypes.
 */
bool can_create( CHAR_DATA *ch, char *argument );


void do_anum( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    BUFFER *buf2;
    bool match = FALSE;
    AREA_DATA *pArea;
    sh_int c_d = 0, c_l = 0, c_v = 0;
    sh_int c_o = 0, c_i = 0, c_e = 0;
    sh_int c_f = 0, c_q = 0, c_a = 0;
    sh_int c_r = 0, c_area = 0;

    if ( argument[0] == '\0' )
    {
	for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	{
	    buf2 = new_buf();
	    // list the area flags -- Bri, 31/12/99
	    if ( IS_SET(pArea->flags, AREA_ARENA) ) {
	        add_buf( buf2, "A" );
		c_a++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_DIRTY) ) {
	        add_buf( buf2, "D" );
		c_d++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_EVERLOAD) ) {
	        add_buf( buf2, "E" );
		c_e++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_FREE) ) {
	        add_buf( buf2, "F" );
		c_f++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_INVIS) ) {
	        add_buf( buf2, "I" );
		c_i++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_LAWLESS) ) {
	        add_buf( buf2, "L" );
		c_l++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_OPEN_SLATHER) ) {
	        add_buf( buf2, "O" );
		c_o++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_QUIET) ) {
		add_buf( buf2, "Q" );
		c_q++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_REVAMP) ) {
		add_buf( buf2, "R" );
		c_r++;
	    } else
		add_buf( buf2, " " );

	    if ( IS_SET(pArea->flags, AREA_VIRTUAL) ) {
	        add_buf( buf2, "V" );
		c_v++;
	    } else
		add_buf( buf2, " " );

	    c_area++;

	    if ( get_olc_level(ch) == OLC_LEVEL_AREA )
	    {
	    sprintf( buf, "%5d  %5d  %s  %-15s  %s\n\r",
		pArea->min_vnum,
		pArea->max_vnum,
		buf_string(buf2),
		pArea->file_name,
		pArea->name );
	    }
	    else
	    {
	    sprintf( buf, "%5d  %5d  %s  %s\n\r",
		pArea->min_vnum,
		pArea->max_vnum,
		buf_string(buf2),
		pArea->name );
	    }
	    
	    send_to_char( buf, ch );
	    free_buf(buf2);
	}
	sprintf( buf, "\n\r%3d areas:   %3d arena,  %3d dirty,   %3d everload,     %3d free,\n\r",
		c_area, c_a, c_d, c_e, c_f );
	send_to_char( buf, ch );
	sprintf( buf, "             %3d invis,  %3d lawless, %3d open_slather, %3d quiet,\n\r",
		c_i, c_l, c_o, c_q );
	send_to_char( buf, ch );
	sprintf( buf, "             %3d revamp, %3d virtual\n\r",
		c_r, c_v );
	send_to_char( buf, ch );
	return;
    }
    
    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
    {
	if ( !str_prefix(argument, pArea->name) )
	{
	    if ( get_olc_level(ch) == OLC_LEVEL_AREA )
	    {
	    sprintf( buf, "%-5d  %-5d  %-15s  %s\n\r"
	                  "  Credits : %s\n\r"
	                  "  Creators: %s\n\r"
	                  "  Flags   : %s\n\r"
                          "  Music   : %s\n\r",
		pArea->min_vnum,
		pArea->max_vnum,
		pArea->file_name,
		pArea->name,
		pArea->credits,
		pArea->creator,
		area_bit_name(pArea->flags),
                pArea->sound_track );
	    }
	    else
	    {
	    sprintf( buf, "%-5d  %-5d  %s\n\r  Credits: %s\n\r  Music:   %s\n\r",
		pArea->min_vnum,
		pArea->max_vnum,
		pArea->name,
		pArea->credits,
                pArea->sound_track );
	    }

	    send_to_char( buf, ch );
	    match = TRUE;
	}
    }
    
    if ( !match )
	send_to_char( "Area not found.\n\r", ch );
}


void do_asave( CHAR_DATA *ch, char *argument )
{
    AREA_DATA *pArea;
    
    if ( argument[0] == '\0' )
    {
	if ( ch->in_room == NULL )
	{
	    send_to_char( "Save which area?\n\r", ch );
	    return;
	}
	
	pArea = ch->in_room->area;
	
	if ( !is_creator(ch, pArea) )
	    send_to_char( "You are not a creator in this area.\n\r", ch );
	else
	{
	    fwrite_area( pArea );
	    
	    send_to_char( "Area saved.\n\r", ch );
	}
	
	return;
    }
    
    if ( !str_cmp(argument, "unsaved") )
    {
	for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	    if ( IS_SET(pArea->flags, AREA_DIRTY)
	    &&   is_creator(ch, pArea) )
		fwrite_area( pArea );

	send_to_char( "All areas saved.\n\r", ch );
	return;
    }
    
    if ( !str_cmp(argument, "all") && get_trust(ch) == MAX_LEVEL )
    {
	for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	    fwrite_area( pArea );
	
	send_to_char( "All areas saved.\n\r", ch );
	return;
    }
    
    send_to_char( "You can't do that!\n\r", ch );
}


void do_unsave( CHAR_DATA *ch, char *argument )
{
    bool first = TRUE;
    AREA_DATA *pArea;
    
    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
    {
	if ( IS_SET(pArea->flags, AREA_DIRTY) 
	&&   is_creator(ch, pArea) )
	{
	    if ( first )
	    {
		send_to_char( "The following areas are unsaved:\n\r", ch );
		first = FALSE;
	    }
	    
	    act( "  $t", ch, pArea->name, NULL, TO_CHAR );
	}
    }
    
    if ( !first )
	send_to_char( "\n\r", ch );
    else
	send_to_char( "All areas are saved.\n\r", ch );

    territory_unsaved( ch );
}


void do_area_credits( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    AREA_DATA *pArea;

    argument = one_argument( argument, arg );
    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	if ( !str_prefix(arg, pArea->name) )
	    break;

    if ( pArea == NULL )
    {
	send_to_char( "Which area?\n\r", ch );
	return;
    }
    
    if ( argument[0] == '\0' )
    {
	sprintf( buf, "Credits: %s\n\r", pArea->credits );
	send_to_char( buf, ch );
	return;
    }
    
    smash_tilde( argument );
    
    free_string( pArea->credits );
    pArea->credits = str_dup( argument );
    
    SET_BIT( pArea->flags, AREA_DIRTY );
    send_to_char( "OK.\n\r", ch );
}


void do_area_name( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    AREA_DATA *pArea;

    argument = one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	if ( ch->in_room == NULL )
	{
	    send_to_char( "Which area?\n\r", ch );
	    return;
	}

	pArea = ch->in_room->area;
	
	argument = arg;
    }
    else
    {
	for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	    if ( !str_prefix(arg, pArea->name) )
		break;
	
	if ( pArea == NULL )
	{
	    send_to_char( "Which area?\n\r", ch );
	    return;
	}
    }
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set the area name to what?\n\r", ch );
	return;
    }
    
    smash_tilde( argument );
    
    free_string( pArea->name );
    pArea->name = str_dup( argument );
    
    SET_BIT( pArea->flags, AREA_DIRTY );
    send_to_char( "OK.\n\r", ch );
}


void do_area_creator( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    AREA_DATA *pArea;

    argument = one_argument( argument, arg );
    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	if ( !str_prefix(arg, pArea->name) )
	    break;
	
    if ( pArea == NULL )
    {
	send_to_char( "Which area?\n\r", ch );
	return;
    }
    
    if ( argument[0] == '\0' )
    {
	sprintf( buf, "Creator: %s\n\r", pArea->creator );
	send_to_char( buf, ch );
	return;
    }
    
    if ( !str_cmp(argument, "none") )
    {
	free_string( pArea->creator );
	pArea->creator = str_dup( "" );
	send_to_char( "This area now has no creator.\n\r", ch );
	SET_BIT( pArea->flags, AREA_DIRTY );
	return;
    }
    
    smash_tilde( argument );

    free_string( pArea->creator );
    sprintf(buf, "Setting %s to be creator of %s\n\r", argument, pArea->name);
    send_to_char (buf,ch);
    pArea->creator = str_dup( argument );
    SET_BIT( pArea->flags, AREA_DIRTY );
    sprintf( buf, "Creator: %s\n\r", pArea->creator );
    send_to_char( buf, ch );
    send_to_char( "OK.\n\r", ch );
}


void do_area_flags( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    AREA_DATA *pArea;
    long flags;
    char type;
    int pos;

    argument = one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
	if ( ch->in_room == NULL )
	{
	    send_to_char( "Which area?\n\r", ch );
	    return;
	}

	pArea = ch->in_room->area;
	
	argument = arg;
    }
    else
    {
	for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
	    if ( !str_prefix(arg, pArea->name) )
		break;
	
	if ( pArea == NULL )
	{
	    send_to_char( "Which area?\n\r", ch );
	    return;
	}
    }
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set what area flags?\n\r", ch );
	return;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {	
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, area_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return;
	}
	
	SET_BIT( flags, pos );
    }

    for ( pos = 0; area_flags[pos].name != NULL; pos++ )
    {
        if ( !area_flags[pos].settable && IS_SET(flags, area_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        area_flags[pos].name );
	    send_to_char( buf, ch );
	    return;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pArea->flags, flags );
	break;
    case '-':
	REMOVE_BIT( pArea->flags, flags );
	break;
    case '=':
	pArea->flags = flags;
	break;
    default:
	pArea->flags = pArea->flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    
    SET_BIT( pArea->flags, AREA_DIRTY );
}


void do_area_sound( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    AREA_DATA *pArea;

    argument = one_argument( argument, arg );
    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
        if ( !str_prefix(arg, pArea->name) )
	    break;
	
    if ( pArea == NULL )
    {
	send_to_char( "Which area?\n\r", ch );
	return;
    }
    
    if ( argument[0] == '\0' )
    {
        send_to_char( "Set sound track to what?\n\r", ch );
        return;
    }

    one_argument( argument, arg );

    free_string( pArea->sound_track );
    if ( str_cmp(arg, "none") )
        pArea->sound_track = str_dup( arg );
    
    SET_BIT( pArea->flags, AREA_DIRTY );
    send_to_char( "OK.\n\r", ch );
}


void do_create( CHAR_DATA *ch, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    AREA_DATA *pArea = NULL;
    int vnum, iHash;
    
    argument = one_argument( argument, arg1 );
    one_argument( argument, arg2 );
    
    if ( arg1[0] == '\0' || arg2[0] == '\0' || !is_number(arg2) )
    {
	send_to_char( "Syntax: create room    <vnum>\n\r"
	              "        create mobile  <vnum>\n\r"
	              "        create object  <vnum>\n\r"
	              "        create program <vnum>\n\r", ch );
	return;
    }
    
    vnum = atoi( arg2 );
    
    for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
    {
	if ( vnum >= pArea->min_vnum && vnum <=pArea->max_vnum )
	{
	    if ( !is_creator(ch, pArea) )
	    {
		send_to_char( "You are not a creator in that area.\n\r", ch );
		return;
	    }
	    
	    break;
	}
    }

    if ( pArea == NULL )
    {
	send_to_char( "That vnum is not valid for any current area.\n\r", ch );
	return;
    }
    
    if ( !str_prefix(arg1, "room") )
    {
	ROOM_INDEX_DATA *pRoomIndex;
	int door;
	
	if ( !can_create(ch, "room") )
	    return;
	
	if ( get_room_index(vnum) != NULL )
	{
	    send_to_char( "There is already a room with that vnum.\n\r", ch );
	    return;
	}
	
	pRoomIndex			= alloc_perm( sizeof(*pRoomIndex) );
	pRoomIndex->owner		= str_dup("");
	pRoomIndex->people		= NULL;
	pRoomIndex->contents		= NULL;
	pRoomIndex->extra_descr		= NULL;
	pRoomIndex->area		= pArea;
	pRoomIndex->vnum		= vnum;
	pRoomIndex->name		= str_dup("An empty room.");
	pRoomIndex->description		= str_dup("");
	pRoomIndex->room_flags		= 0;
	pRoomIndex->sector_type		= SECT_CITY;
	pRoomIndex->light		= 0;
	
	for ( door = 0; door <= 11; door++ )
	    pRoomIndex->exit[door]	= NULL;

	/* defaults */
	pRoomIndex->heal_rate 		= 100;
	pRoomIndex->mana_rate 		= 100;
	
	SET_BIT( pArea->flags, AREA_DIRTY );

	iHash				= vnum % MAX_KEY_HASH;
	pRoomIndex->next		= room_index_hash[iHash];
	room_index_hash[iHash]		= pRoomIndex;
	top_room++;
    }
    else if ( !str_prefix(arg1, "mobile") )
    {
	MOB_INDEX_DATA *pMobIndex;
	RESET_DATA *pReset;
	CHAR_DATA *pMobile;

	if ( !can_create(ch, "mobile") )
	    return;

	if ( ch->in_room == NULL
	||   ch->in_room->area != pArea )
	{
	    send_to_char( "You must be in that area to create a mobile.\n\r", ch );
	    return;
	}

	if ( get_mob_index(vnum) != NULL )
	{
	    send_to_char( "There is already a mobile with that vnum.\n\r", ch );
	    return;
	}
	
	pMobIndex			= alloc_perm( sizeof(*pMobIndex) );
	pMobIndex->vnum			= vnum;
	pMobIndex->new_format		= TRUE;
	pMobIndex->area			= pArea;

	pMobIndex->player_name		= str_dup( "generic mobile" );
	pMobIndex->short_descr		= str_dup( "a generic mobile" );
	pMobIndex->long_descr		= str_dup( "A generic mobile is here.\n\r" );
	pMobIndex->description		= str_dup( "" );
	pMobIndex->race			= race_lookup( "human" );
	pMobIndex->act			= ACT_IS_NPC;
	pMobIndex->aggression		= 0;
	pMobIndex->affected_by		= 0;
        pMobIndex->affected_by2         = 0;
	pMobIndex->pShop		= NULL;
	pMobIndex->alignment		= 0;
	pMobIndex->group		= 0;
	pMobIndex->level		= 1;
	pMobIndex->hitroll		= 0;
	
	pMobIndex->hit[DICE_NUMBER]	= 1;
	pMobIndex->hit[DICE_TYPE]	= 6;
	pMobIndex->hit[DICE_BONUS]	= 0;
	pMobIndex->mana[DICE_NUMBER]	= 1;
	pMobIndex->mana[DICE_TYPE]	= 6;
	pMobIndex->mana[DICE_BONUS]	= 0;
	pMobIndex->damage[DICE_NUMBER]	= 1;
	pMobIndex->damage[DICE_TYPE]	= 6;
	pMobIndex->damage[DICE_BONUS]	= 0;
	pMobIndex->dam_type		= 5;	/* claw */
	
	pMobIndex->ac[AC_PIERCE]	= 100;
	pMobIndex->ac[AC_BASH]		= 100;
	pMobIndex->ac[AC_SLASH]		= 100;
	pMobIndex->ac[AC_EXOTIC]	= 100;
	
	pMobIndex->off_flags		= 0;
	pMobIndex->imm_flags		= 0;
	pMobIndex->res_flags		= 0;
	pMobIndex->vuln_flags		= 0;

	pMobIndex->start_pos		= POS_STANDING;
	pMobIndex->default_pos		= POS_STANDING;
	pMobIndex->sex			= 1;
	
	pMobIndex->wealth		= 0;
	
	pMobIndex->form			= race_table[pMobIndex->race].form;
	pMobIndex->parts		= race_table[pMobIndex->race].parts;
	
	pMobIndex->size			= SIZE_MEDIUM;
	pMobIndex->material		= str_dup( "0" );

        iHash				= vnum % MAX_KEY_HASH;
        pMobIndex->next			= mob_index_hash[iHash];
        mob_index_hash[iHash]		= pMobIndex;
        top_mob_index++;
        
        kill_table[URANGE(0, pMobIndex->level, MAX_LEVEL-1)].number++;
        
	pMobile = create_mobile( pMobIndex );
	char_to_room( pMobile, ch->in_room );

	pReset 		= create_reset( pArea, NULL );
	pReset->command = 'M';
	pReset->arg1	= pMobIndex->vnum;
	pReset->arg2	= 1;
	pReset->arg3	= ch->in_room->vnum;
	pReset->arg4	= 1;
    }
    else if ( !str_prefix(arg1, "object") )
    {
	OBJ_INDEX_DATA *pObjIndex;
	RESET_DATA *pReset;
	OBJ_DATA *pObject;

	if ( !can_create(ch, "object") )
	    return;

	if ( ch->in_room == NULL
	||   ch->in_room->area != pArea )
	{
	    send_to_char( "You must be in that area to create an object.\n\r", ch );
	    return;
	}

	if ( get_obj_index(vnum) != NULL )
	{
	    send_to_char( "There is already an object with that vnum.\n\r", ch );
	    return;
	}
	
	pObjIndex			= alloc_perm( sizeof(*pObjIndex) );
	pObjIndex->vnum			= vnum;
	pObjIndex->new_format		= TRUE;
	pObjIndex->area			= pArea;
	pObjIndex->reset_num		= 0;
	
	pObjIndex->name			= str_dup( "generic object" );
	pObjIndex->short_descr		= str_dup( "a generic object" );
	pObjIndex->description		= str_dup( "A generic object sits here.\n\r" );
	pObjIndex->material		= str_dup( "0" );
	pObjIndex->item_type		= ITEM_TRASH;
	pObjIndex->extra_flags		= 0;
	pObjIndex->wear_flags		= 0;
	pObjIndex->restrictions		= 0;
	pObjIndex->value[0]		= 0;
	pObjIndex->value[1]		= 0;
	pObjIndex->value[2]		= 0;
	pObjIndex->value[3]		= 0;
	pObjIndex->value[4]		= 0;
	pObjIndex->level		= 1;
	pObjIndex->weight		= 10;
	pObjIndex->cost			= 0;
	pObjIndex->condition		= 0;

	iHash			= vnum % MAX_KEY_HASH;
	pObjIndex->next		= obj_index_hash[iHash];
	obj_index_hash[iHash]	= pObjIndex;
	top_obj_index++;
	
	pObject = create_object( pObjIndex, get_trust(ch), FALSE );
	obj_to_room( pObject, ch->in_room );

	pReset 		= create_reset( pArea, NULL );
	pReset->command = 'O';
	pReset->arg1	= pObjIndex->vnum;
	pReset->arg2	= 1;
	pReset->arg3	= ch->in_room->vnum;
	pReset->arg4	= 1;
    }
    else if ( !str_prefix(arg1, "program") )
    {
	MPROG_CODE *pMProg;

	if ( !can_create(ch, "program") )
	    return;

	if ( ch->in_room == NULL
	||   ch->in_room->area != pArea )
	{
	    send_to_char( "You must be in that area to create a program for it.\n\r", ch );
	    return;
	}

	if ( get_mprog_index(vnum) != NULL )
	{
	    send_to_char( "There is already a program with that vnum.\n\r", ch );
	    return;
	}
	
	pMProg			= alloc_perm( sizeof(*pMProg) );
	pMProg->vnum		= vnum;
	pMProg->code		= str_dup( "" );

	if ( mprog_list == NULL )
	    mprog_list		= pMProg;
	else
	{
	    pMProg->next	= mprog_list;
	    mprog_list 		= pMProg;
	}
    }
    else
    {
	send_to_char( "Invalid sub-command.\n\r", ch );
	return;
    }

    SET_BIT( pArea->flags, AREA_DIRTY );
    send_to_char( "OK.\n\r", ch );
}


bool is_creator( CHAR_DATA *ch, AREA_DATA *area )
{
    if ( get_olc_level(ch) == OLC_LEVEL_AREA )
	return TRUE;

    if ( area == NULL )
	return FALSE;

    if ( is_exact_name(true_name(ch), area->creator) )
	return TRUE;

    if ( IS_IMMORTAL(ch)
    &&   IS_SET(area->flags, AREA_OPEN_SLATHER) )
	return TRUE;

    return FALSE;
}


bool can_create( CHAR_DATA *ch, char *argument )
{
    int i;
    
    for ( i = 0; !IS_NULLSTR(create_levels[i].name); i++ )
    {
	if ( !str_cmp(create_levels[i].name, argument) )
	    if ( get_olc_level(ch) >= create_levels[i].olc_level )
		return TRUE;
	    else
	    {
		send_to_char( "You don't have the necessary permissions to create that.\n\r", ch );
		return FALSE;
	    }
	}
    
    return FALSE;
}


void do_areset( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    AREA_DATA *pArea;

    argument = one_argument( argument, arg );
    if ( arg[0] == '\0' )
    {
        if ( ch->in_room == NULL )
        {
            send_to_char( "Which area?\n\r", ch );
            return;
        }

        pArea = ch->in_room->area;

        argument = arg;
    }
    else
    {
        for ( pArea = area_first; pArea != NULL; pArea = pArea->next )
            if ( !str_prefix(arg, pArea->name) )
                break;

        if ( pArea == NULL )
        {
            send_to_char( "Which area?\n\r", ch );
            return;
        }
    }

    if ( !is_creator(ch, pArea) )
    {
	send_to_char( "You need creator privileges to force a reset.\n\r", ch );
	return;
    }

    reset_area( pArea );

    send_to_char( "OK.\n\r", ch );
    return;
}
