/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#include <sys/resource.h>
#endif
#include "merc.h"
#include "tables.h"
#include "recycle.h"
#include "lookup.h"
#include "olc.h"


/*
 * Type definitions.
 */
typedef	bool OBJECT_FUN	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
				OBJ_INDEX_DATA *pObjIndex,
				char *argument ) );

struct object_command
{
    char *	name;
    OBJECT_FUN	*fun;
    bool	update;
    int		olc_level;
};


/*
 * Global function prototypes.
 */
int     flag_lookup     args( ( const char *name,
                                const struct flag_type *flag_table ) );

/*
 * Local function prototypes.
 */
bool do_object_level		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_name		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_cost		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_weight		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_short		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_long		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_worn		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_wear_self	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_wear_room	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_remove_self	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_remove_room	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_type		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_apply		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_extra		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_extend		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_wear		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_restritctions	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_clone		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_purge		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_put		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_auto		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_food		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_liquid		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_light		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_armor		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_furniture	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_use		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_spell		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_container	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_weapon		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_quiver		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_arrow		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_poison		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_portal		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_focus		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_money		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_restrictions	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_limit		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_resets		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_fit		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_affect		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_immune		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_resist		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_vuln		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_skill            args( ( CHAR_DATA *ch, AREA_DATA *pArea,
                                        OBJ_INDEX_DATA *pObjIndex,
                                        char *argument ) );
bool do_object_program		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_description	args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );
bool do_object_message		args( ( CHAR_DATA *ch, AREA_DATA *pArea,
					OBJ_INDEX_DATA *pObjIndex,
					char *argument ) );


/*
 * The object command table.
 */
const struct object_command object_commands[] =
{
    { "level",		do_object_level,	TRUE,	2	},
    { "name",		do_object_name,		TRUE,	1	},
    { "cost",		do_object_cost,		TRUE,	2	},
    { "weight",		do_object_weight,	TRUE,	2	},
    { "short",		do_object_short,	TRUE,	3	},
    { "long",		do_object_long,		TRUE,	3	},
    { "wear",		do_object_wear,		TRUE,	2	},
    { "restrictions",	do_object_restrictions,	FALSE,	2	},
    { "worn",		do_object_worn,		FALSE,	1	},
    { "wear_self",	do_object_wear_self,	FALSE,	1	},
    { "wear_room",	do_object_wear_room,	FALSE,	1	},
    { "remove_self",	do_object_remove_self,	FALSE,	1	},
    { "remove_room",	do_object_remove_room,	FALSE,	1	},
    { "type",		do_object_type,		TRUE,	2	},
    { "apply",		do_object_apply,	TRUE,	2	},
    { "extra",		do_object_extra,	TRUE,	2	},
    { "extended",	do_object_extend,	FALSE,	1	},
    { "clone",		do_object_clone,	FALSE,	1	},
    { "purge",		do_object_purge,	FALSE,	1	},
    { "put",		do_object_put,		FALSE,	1	},
    { "auto",		do_object_auto,		TRUE,	2	},
    { "food",		do_object_food,		TRUE,	2	},
    { "liquid",		do_object_liquid,	TRUE,	2	},
    { "light",		do_object_light,	TRUE,	2	},
    { "armor",		do_object_armor,	TRUE,	2	},
    { "furniture",	do_object_furniture,	TRUE,	2	},
    { "use",		do_object_use,		TRUE,	2	},
    { "spell",		do_object_spell,	TRUE,	2	},
    { "container",	do_object_container,	TRUE,	2	},
    { "weapon",		do_object_weapon,	TRUE,	2	},
    { "quiver",		do_object_quiver,	TRUE,	2	},
    { "arrow",		do_object_arrow,	TRUE,	2	},
    { "poison",		do_object_poison,	TRUE,	2	},
    { "portal",		do_object_portal,	TRUE,	2	},
    { "focus",		do_object_focus,	TRUE,	2	},
    { "money",		do_object_money,	TRUE,	4	},
    { "limit",		do_object_limit,	FALSE,	1	},
    { "resets",		do_object_resets,	FALSE,	1	},
    { "fit",		do_object_fit,		FALSE,	2	},
    { "affect",		do_object_affect,	FALSE,	5	},
    { "immune",		do_object_immune,	FALSE,	5	},
    { "resist",		do_object_resist,	FALSE,	5	},
    { "vuln",		do_object_vuln,		FALSE,	5	},
    { "skill",          do_object_skill,        FALSE,  5,      },
    { "program",	do_object_program,	FALSE,	4	},
    { "description",	do_object_description,	FALSE,	1	},
    { "message",	do_object_message,	FALSE,	1	},
    { NULL }
};


/*
 * Automatic values.
 */
const struct weapon_auto weapon_autos[] =
{
    {  1,  6 }, {  1,  6 }, {  1,  6 },
    {  1,  8 }, {  1,  8 }, {  1,  8 },
    {  2,  5 }, {  2,  5 }, {  2,  5 },
    {  2,  6 }, {  2,  6 }, {  2,  6 },
    {  2,  7 }, {  2,  7 }, {  2,  7 },
    {  2,  8 }, {  2,  8 }, {  2,  8 },
    {  3,  6 }, {  3,  6 }, {  3,  6 },
    {  3,  7 }, {  3,  7 }, {  3,  7 },
    {  2, 11 }, {  2, 11 }, {  2, 11 },
    {  3,  8 }, {  3,  8 }, {  3,  8 },
    {  6,  5 }, {  6,  5 }, {  6,  5 },
    {  4,  8 }, {  4,  8 }, {  4,  8 },
    {  2, 17 }, {  2, 17 }, {  2, 17 },
    {  4,  9 }, {  4,  9 }, {  4,  9 },
    {  2, 19 }, {  2, 19 }, {  2, 19 },
    {  4, 10 }, {  4, 10 }, {  4, 10 },
    {  6,  7 }, {  6,  7 }, {  6,  7 },
    {  4, 11 }, {  4, 11 }, {  4, 11 },
    {  2, 23 }, {  2, 23 }, {  2, 23 },
    {  4, 12 }, {  4, 12 }, {  4, 12 }
};


bool get_two_dice( char *argument, int *number, int *dice )
{
    char *current;
    
    for ( current = argument; *current != '\0'; current++ )
    {
	if ( *current == 'd' || *current == 'D' )
	{
	    *current = '\0';
	    
	    if ( !is_number(argument) )
		return FALSE;
		
	    *number = atoi( argument );
	    
	    current++;
	    break;
	}
	
	if ( !isdigit(*current) )
	    return FALSE;
    }
    
    if ( !is_number(current) )
	return FALSE;
	
    *dice = atoi( current );
    return TRUE;
}


USE_MSG_DATA *find_use_data( OBJ_INDEX_DATA *pObjIndex, sh_int tag )
{
    USE_MSG_DATA *current;
    
    for ( current = pObjIndex->use_msgs; current != NULL; current = current->next )
    {
	if ( current->tag == tag )
	    return current;
    }
    
    return NULL;
}


void extract_obj_use_msg( OBJ_INDEX_DATA *pObjIndex, sh_int tag )
{
    if ( pObjIndex->use_msgs == NULL )
	return;
    else if ( pObjIndex->use_msgs->tag == tag )
    {
	USE_MSG_DATA *tail = pObjIndex->use_msgs->next;
	
	free_use_msg( pObjIndex->use_msgs );
	
	pObjIndex->use_msgs = tail;
    }
    else
    {
	USE_MSG_DATA *current;
	
	for ( current = pObjIndex->use_msgs; current != NULL; current = current->next )
	{
	    if ( current->next == NULL )
	    {
		bug( "extract_obj_use_msg: use message not found.", 0 );
		break;
	    }
	
	    if ( current->next->tag == tag )
	    {
		USE_MSG_DATA *temp = current->next;
		
		current->next = current->next->next;
		
		free_use_msg( temp );
	    }
	}
    }
}


void do_object( CHAR_DATA *ch, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    OBJ_INDEX_DATA *pObjIndex;
    AREA_DATA *pArea;
    int command;
    
    if ( ch->in_room == NULL )
    {
	send_to_char( "You need to be in a room before you try doing that.\n\r", ch );
	return;
    }
    
    pArea = ch->in_room->area;

    if ( !is_creator(ch, pArea) )
    {
	send_to_char( "You are not a creator in this area.\n\r", ch );
	return;
    }

    if ( argument[0] == '\0' )
    {
	send_to_char( "Which object to you want to edit?\n\r", ch );
	return;
    }

    argument = one_argument( argument, arg );
    if ( is_number(arg) )
    {
	if ( (pObjIndex = get_obj_index(atoi(arg))) == NULL )
	{
	    send_to_char( "No object with that vnum found.\n\r", ch );
	    return;
	}
	
	if ( pObjIndex->vnum < pArea->min_vnum
	||   pObjIndex->vnum > pArea->max_vnum )
	{
	    send_to_char( "That object isn't a part of this area.\n\r", ch );
	    return;
	}
    }
    else
    {
	int vnum;
	
	pObjIndex = NULL;
	
	for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
	{
	    if ( (pObjIndex = get_obj_index(vnum)) != NULL
	    &&   is_name(arg, pObjIndex->name) )
		break;
	    
	    pObjIndex = NULL;
	}
	
	if ( pObjIndex == NULL )
	{
	    send_to_char( "No object with that name found in this area.\n\r", ch );
	    return;
	}
    }

    argument = one_argument( argument, arg );

    for ( command = 0; object_commands[command].name != NULL; command++ )
    {
	if ( LOWER(arg[0]) == LOWER(object_commands[command].name[0])
	&&   !str_prefix(arg, object_commands[command].name))
	{
	    if ( get_olc_level(ch) < object_commands[command].olc_level )
	    {
		send_to_char( 
		    "You do not have sufficient OLC privileges to do that.\n\r", 
		    ch );
		return;
	    }
	    
	    if ( (*object_commands[command].fun) 
	            ( ch, pArea, pObjIndex, argument ) )
	    {
		SET_BIT( pArea->flags, AREA_DIRTY );
		
		if ( object_commands[command].update )
		{
		    OBJ_DATA *current;
		    AFFECT_DATA *paf;
		    
		    for ( current = pObjIndex->instances;
		          current != NULL;
		          current = current->next_instance )
		    {
			free_string( current->name );
			free_string( current->short_descr );
			free_string( current->description );
			
			current->name		= str_dup( pObjIndex->name );
			current->short_descr	= str_dup( pObjIndex->short_descr );
			current->description	= str_dup( pObjIndex->description );
			current->level		= pObjIndex->level;
			current->item_type	= pObjIndex->item_type;
			current->extra_flags	= pObjIndex->extra_flags;
			current->wear_flags	= pObjIndex->wear_flags;
			current->value[0]	= pObjIndex->value[0];
			current->value[1]	= pObjIndex->value[1];
			current->value[2]	= pObjIndex->value[2];
			current->value[3]	= pObjIndex->value[3];
			current->value[4]	= pObjIndex->value[4];
			current->weight		= pObjIndex->weight;
			current->cost		= pObjIndex->cost;

			for ( paf = pObjIndex->affected; 
			      paf != NULL; 
			      paf = paf->next ) 
			    if ( paf->location == APPLY_SPELL_AFFECT )
				affect_to_obj( current, paf );
		    }
		}
	    }

	    return;
	}
    }

    send_to_char( "That's not a sub-command.\n\r", ch );
}


bool do_object_level( 
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    int level;
    
    if ( !is_number(argument) )
    {
	send_to_char( "That's not a valid level.\n\r", ch );
	return FALSE;
    }
    
    level = atoi( argument );
    
    if ( level <= 0 )
    {
	send_to_char( "The level must be greater than zero.\n\r", ch );
	return FALSE;
    }
    
    if ( level > MAX_LEVEL )
    {
	send_to_char( "That level is too high.\n\r", ch );
	return FALSE;
    }
    
    pObjIndex->level = level;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_name( 
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object name to what?\n\r", ch );
	return FALSE;
    }
    
    smash_tilde( argument );
    
    free_string( pObjIndex->name );
    pObjIndex->name = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_cost( 
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    int cost;
    
    if ( !is_number(argument) )
    {
	send_to_char( "That's not a valid cost.\n\r", ch );
	return FALSE;
    }
    
    cost = atoi( argument );
    
    if ( cost <= 0 )
    {
	send_to_char( "The cost must be greater than zero.\n\r", ch );
	return FALSE;
    }
    
    pObjIndex->cost = cost;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_weight( 
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    int weight;
    
    if ( !is_number(argument) )
    {
	send_to_char( "That's not a valid weight.\n\r", ch );
	return FALSE;
    }
    
    weight = atoi( argument );
    
    if ( weight <= 0 )
    {
	send_to_char( "The weight must be greater than zero.\n\r", ch );
	return FALSE;
    }
    
    pObjIndex->weight = weight;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_short( 
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object short description to what?\n\r", ch );
	return FALSE;
    }
    
    smash_tilde( argument );
    
    free_string( pObjIndex->short_descr );
    pObjIndex->short_descr = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_long(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object long description to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !str_cmp(argument, "NULL") )
    {
	free_string( pObjIndex->description );
	pObjIndex->description = str_dup( "" );

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    
    free_string( pObjIndex->description );
    pObjIndex->description = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_worn(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    USE_MSG_DATA *use_msg;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object wear location name to what?\n\r", ch );
	return FALSE;
    }

    use_msg = find_use_data( pObjIndex, USE_WEAR_LOC );
    
    if ( !str_cmp(argument, "NULL") )
    {
	if ( use_msg != NULL )
	    extract_obj_use_msg( pObjIndex, USE_WEAR_LOC );

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    
    if ( use_msg == NULL )
    {
	use_msg = new_use_msg();
	use_msg->next = pObjIndex->use_msgs;
	pObjIndex->use_msgs = use_msg;
	
	use_msg->tag = USE_WEAR_LOC;
    }

    free_string( use_msg->msg );
    use_msg->msg = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_wear_self(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    USE_MSG_DATA *use_msg;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object wear self message to what?\n\r", ch );
	return FALSE;
    }
    
    use_msg = find_use_data( pObjIndex, USE_WEAR_SELF );

    if ( !str_cmp(argument, "NULL") )
    {
	if ( use_msg != NULL )
	    extract_obj_use_msg( pObjIndex, USE_WEAR_SELF );

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
        
    if ( use_msg == NULL )
    {
	use_msg = new_use_msg();
	use_msg->next = pObjIndex->use_msgs;
	pObjIndex->use_msgs = use_msg;

	use_msg->tag = USE_WEAR_SELF;
    }

    free_string( use_msg->msg );
    use_msg->msg = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_wear_room(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    USE_MSG_DATA *use_msg;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object wear room message to what?\n\r", ch );
	return FALSE;
    }

    use_msg = find_use_data( pObjIndex, USE_WEAR_ROOM );
    
    if ( !str_cmp(argument, "NULL") )
    {
	if ( use_msg != NULL )
	    extract_obj_use_msg( pObjIndex, USE_WEAR_ROOM );

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }

    if ( use_msg == NULL )
    {
	use_msg = new_use_msg();
	use_msg->next = pObjIndex->use_msgs;
	pObjIndex->use_msgs = use_msg;

	use_msg->tag = USE_WEAR_ROOM;
    }
        
    free_string( use_msg->msg );
    use_msg->msg = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_remove_self(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    USE_MSG_DATA *use_msg;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object remove self message to what?\n\r", ch );
	return FALSE;
    }
    
    use_msg = find_use_data( pObjIndex, USE_REMOVE_SELF );
    
    if ( !str_cmp(argument, "NULL") )
    {
	if ( use_msg != NULL )
	    extract_obj_use_msg( pObjIndex, USE_REMOVE_SELF );

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }

    if ( use_msg == NULL )
    {
	use_msg = new_use_msg();
	use_msg->next = pObjIndex->use_msgs;
	pObjIndex->use_msgs = use_msg;

	use_msg->tag = USE_REMOVE_SELF;
    }

    free_string( use_msg->msg );
    use_msg->msg = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_remove_room(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    USE_MSG_DATA *use_msg;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object remove room message to what?\n\r", ch );
	return FALSE;
    }

    use_msg = find_use_data( pObjIndex, USE_REMOVE_ROOM );
    
    if ( !str_cmp(argument, "NULL") )
    {
	if ( use_msg != NULL )
	    extract_obj_use_msg( pObjIndex, USE_REMOVE_ROOM );

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }
    
    if ( use_msg == NULL )
    {
	use_msg = new_use_msg();
	use_msg->next = pObjIndex->use_msgs;
	pObjIndex->use_msgs = use_msg;
	
	use_msg->tag = USE_REMOVE_ROOM;
    }

    free_string( use_msg->msg );
    use_msg->msg = str_dup( argument );
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_type(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    USE_MSG_DATA *use_msg, *use_msg_next;
    int type;
    int i;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object type to what?\n\r", ch );
	return FALSE;
    }
    
    type = item_lookup( argument );
    if ( type == -1 )
    {
	send_to_char( "That's not a valid item type.\n\r", ch );
	return FALSE;
    }
    
    pObjIndex->item_type = type;
    
    pObjIndex->value[0] = 0;
    pObjIndex->value[1] = 0;
    pObjIndex->value[2] = 0;
    pObjIndex->value[3] = 0;
    pObjIndex->value[4] = 0;
    
    switch ( pObjIndex->item_type )
    {
    case ITEM_FURNITURE:
	pObjIndex->value[3] = 100;
	pObjIndex->value[4] = 100;
	break;
    case ITEM_CONTAINER:
	pObjIndex->value[4] = 100;
	break;
    case ITEM_PORTAL:
	pObjIndex->value[2] = GATE_NORMAL_EXIT;
	break;
    }
    
    for ( use_msg = pObjIndex->use_msgs; use_msg != NULL; use_msg = use_msg_next )
    {
	use_msg_next = use_msg->next;
	
	for ( i=0; !IS_NULLSTR(use_msgs[i].type); i++ )
	{
	    if ( use_msgs[i].tag == use_msg->tag
	    &&   use_msgs[i].item_type != 0
	    &&   use_msgs[i].item_type != pObjIndex->item_type )
		extract_obj_use_msg( pObjIndex, use_msg->tag );
	}
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_apply(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char loc_arg[MAX_INPUT_LENGTH], mod_arg[MAX_INPUT_LENGTH];
    int location, modifier;
    AFFECT_DATA *paf;
    
    argument = one_argument( argument, loc_arg );
    one_argument( argument, mod_arg );
    
    if ( loc_arg[0] == '\0' || mod_arg[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> apply <location> <modifier>\n\r", ch );
	return FALSE;
    }
    
    location = apply_lookup( loc_arg );
    if ( location == APPLY_NONE )
    {
	send_to_char( "Invalid apply location.\n\r", ch );
	return FALSE;
    }
    
    if ( !is_number(mod_arg) )
    {
	send_to_char( "The modifier must be numeric.\n\r", ch );
	return FALSE;
    }
    
    modifier = atoi( mod_arg );
    
    if ( modifier == 0 )
    {
	AFFECT_DATA *current, *previous;
	
	current = previous = pObjIndex->affected;
	
	while ( current )
	{
	    if ( current->location == location
	    &&   current->bitvector == 0 )
	    {
		if ( current == previous )
		    pObjIndex->affected = current->next;
		else
		    previous->next = current->next;
		
		free_affect( current );
		
		send_to_char( "OK.\n\r", ch );
		return TRUE;
	    }
	    
	    previous = current;
	    current = current->next;
	}
	
	send_to_char( "Apply not found.\n\r", ch );
	return FALSE;
    }

    for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->location == location && paf->bitvector == 0 )
	    break;
    }
    
    if ( paf == NULL )
    {
	paf			= new_affect();
	paf->next		= pObjIndex->affected;
	pObjIndex->affected	= paf;
    }

    paf->where			= TO_OBJECT;
    paf->type			= -1;
    paf->level			= pObjIndex->level;
    paf->duration		= -1;
    paf->location		= location;
    paf->modifier		= modifier;
    paf->bitvector		= 0;
    paf->procNo			= 0;
    paf->flags			= 0;
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_extra(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> extra +<flag list>\n\r"
	              "        obj <vnum> extra -<flag list>\n\r"
	              "        obj <vnum> extra =<flag list>\n\r"
	              "        obj <vnum> extra  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, extra_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; extra_flags[pos].name != NULL; pos++ )
    {
        if ( !extra_flags[pos].settable && IS_SET(flags, extra_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        extra_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pObjIndex->extra_flags, flags );
	break;
    case '-':
	REMOVE_BIT( pObjIndex->extra_flags, flags );
	break;
    case '=':
	pObjIndex->extra_flags = flags;
	break;
    default:
	pObjIndex->extra_flags = pObjIndex->extra_flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_extend(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    EXTRA_DESCR_DATA *ed;
    
    smash_tilde( argument );

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> extended <key> <description>\n\r"
	              "        obj <vnum> extended <key> NULL\n\r", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( !strcmp(argument, "NULL") )
    {
	ed = find_extra( pObjIndex->extra_descr, arg );
	if ( ed == NULL )
	{
	    send_to_char( "Extra description not found.\n\r", ch );
	    return FALSE;
	}
	else
	{
	    remove_extra( &pObjIndex->extra_descr, ed );
	    send_to_char( "Extra description removed.\n\r", ch );
	    return TRUE;
	}
    }
    else
    {
	ed = find_extra( pObjIndex->extra_descr, arg );
	if ( ed != NULL )
	    return edit_desc( ch, &ed->description, argument );
	
	ed = new_extra_descr();
	ed->next = pObjIndex->extra_descr;
	pObjIndex->extra_descr = ed;
	
	ed->keyword = str_dup( arg );
	return edit_desc( ch, &ed->description, argument );
    }
}


bool do_object_wear(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> wear +<flag list>\n\r"
	              "        obj <vnum> wear -<flag list>\n\r"
	              "        obj <vnum> wear =<flag list>\n\r"
	              "        obj <vnum> wear  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, wear_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; wear_flags[pos].name != NULL; pos++ )
    {
        if ( !wear_flags[pos].settable && IS_SET(flags, wear_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        wear_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pObjIndex->wear_flags, flags );
	break;
    case '-':
	REMOVE_BIT( pObjIndex->wear_flags, flags );
	break;
    case '=':
	pObjIndex->wear_flags = flags;
	break;
    default:
	pObjIndex->wear_flags = pObjIndex->wear_flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_restrictions(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> restrictions +<flag list>\n\r"
	              "        obj <vnum> restrictions -<flag list>\n\r"
	              "        obj <vnum> restrictions =<flag list>\n\r"
	              "        obj <vnum> restrictions  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, restriction_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; restriction_flags[pos].name != NULL; pos++ )
    {
        if ( !restriction_flags[pos].settable
        &&   IS_SET(flags, restriction_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        restriction_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pObjIndex->restrictions, flags );
	break;
    case '-':
	REMOVE_BIT( pObjIndex->restrictions, flags );
	break;
    case '=':
	pObjIndex->restrictions = flags;
	break;
    default:
	pObjIndex->restrictions = pObjIndex->restrictions ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_clone(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    RESET_DATA *pReset;
    OBJ_DATA *pObject;
    
    pObject = create_object( pObjIndex, get_trust(ch), FALSE );
    obj_to_room( pObject, ch->in_room );
    
    pReset		= create_reset( pArea, NULL );
    pReset->command	= 'O';
    pReset->arg1	= pObjIndex->vnum;
    pReset->arg3	= ch->in_room->vnum;

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}



bool do_object_purge(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    RESET_DATA *pReset;

    pReset = find_reset( pArea, 'O', pObjIndex->vnum, ch->in_room->vnum );
    if ( pReset == NULL )
    {
	send_to_char( "There was no reset to remove!\n\r", ch );
	return FALSE;
    }
    
    remove_reset( pArea, pReset );

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_put(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    RESET_DATA *pContainerReset, *pPutReset, *pObjReset;
    OBJ_INDEX_DATA *pContainerIndex;
    OBJ_DATA *pObject, *pContainer;
    char arg[MAX_INPUT_LENGTH];

    if ( argument[0] == '\0' )
    {
	send_to_char( "Put into what?\n\r", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    if ( is_number(arg) )
    {
	if ( (pContainerIndex = get_obj_index(atoi(arg))) == NULL )
	{
	    send_to_char( "No object with that vnum found.\n\r", ch );
	    return FALSE;
	}
	
	if ( pContainerIndex->vnum < pArea->min_vnum
	||   pContainerIndex->vnum > pArea->max_vnum )
	{
	    send_to_char( "That object ins't a part of this area.\n\r", ch );
	    return FALSE;
	}
    }
    else
    {
	int vnum;
	
	pContainerIndex = NULL;
	
	for ( vnum = pArea->min_vnum; vnum <= pArea->max_vnum; vnum++ )
	{
	    if ( (pContainerIndex = get_obj_index(vnum)) != NULL
	    &&   is_name(arg, pContainerIndex->name) )
		break;
	    
	    pContainerIndex = NULL;
	}
	
	if ( pContainerIndex == NULL )
	{
	    send_to_char( "No object with that name found in this area.\n\r", ch );
	    return FALSE;
	}
    }
    
    pContainerReset = find_reset( pArea, 'O', pContainerIndex->vnum,
                                  ch->in_room->vnum );
    if ( pContainerReset == NULL )
    {
	send_to_char( "That container doesn't reset into this room.\n\r", ch );
	return FALSE;
    }

    for ( pPutReset = pContainerReset->next; 
          pPutReset != NULL; 
          pPutReset = pPutReset->next )
    {
	if ( pPutReset->command == 'P'
	&&   pPutReset->arg3 == pContainerIndex->vnum
	&&   pPutReset->arg1 == pObjIndex->vnum )
	    break;
    }
    
    if ( pPutReset == NULL )
    {
	pPutReset		= create_reset( pArea, pContainerReset );
	pPutReset->command	= 'P';
	pPutReset->arg1		= pObjIndex->vnum;
	pPutReset->arg2		= 6;
	pPutReset->arg3		= pContainerIndex->vnum;
	pPutReset->arg4		= 1;
    }
    else
	pPutReset->arg4++;

    pObjReset = find_reset( pArea, 'O', pObjIndex->vnum, ch->in_room->vnum );
    if ( pObjReset != NULL )
	remove_reset( pArea, pObjReset );

    for ( pContainer = ch->in_room->contents; 
          pContainer != NULL; 
          pContainer = pContainer->next_content )
    {
	if ( pContainer->pIndexData == pContainerIndex )
	    break;
    }

    if ( pContainer )
    {
	for ( pObject = ch->in_room->contents;
	      pObject != NULL; 
	      pObject = pObject->next_content )
	{
	    if ( pObject->pIndexData == pObjIndex
	    &&   pObject != pContainer )
	    {
		obj_from_room( pObject );
		obj_to_obj( pObject, pContainer );
		break;
	    }
	}
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_auto(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    switch ( pObjIndex->item_type )
    {
    case ITEM_WEAPON:
	pObjIndex->value[1] = weapon_autos[pObjIndex->level - 1].number;
	pObjIndex->value[2] = weapon_autos[pObjIndex->level - 1].type;

	switch ( pObjIndex->value[0] )
	{
	default:		pObjIndex->weight = 100;	break;
	case WEAPON_SWORD:	pObjIndex->weight = 50;		break;
	case WEAPON_DAGGER:	pObjIndex->weight = 15;		break;
	case WEAPON_SPEAR:	pObjIndex->weight = 80;		break;
	case WEAPON_MACE:	pObjIndex->weight = 80;		break;
	case WEAPON_AXE:	pObjIndex->weight = 60;		break;
	case WEAPON_FLAIL:	pObjIndex->weight = 80;		break;
	case WEAPON_WHIP:	pObjIndex->weight = 50;		break;
	case WEAPON_POLEARM:	pObjIndex->weight = 120;	break;
	}

	if ( IS_SET(pObjIndex->value[4], WEAPON_TWO_HANDS) )
	    pObjIndex->weight *= 2;
	break;
    case ITEM_ARMOR:
	if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_FEET) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 12;
	    pObjIndex->value[3] = pObjIndex->level / 16;
	    
	    pObjIndex->weight = 40;
	}
	else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_HANDS)
	||   IS_SET(pObjIndex->wear_flags, ITEM_WEAR_WRIST) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 12;
	    pObjIndex->value[3] = pObjIndex->level / 16;

	    pObjIndex->weight = 8;
	}
	else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_NECK)
	||        IS_SET(pObjIndex->wear_flags, ITEM_WEAR_WAIST) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 11;
	    pObjIndex->value[3] = pObjIndex->level / 15;

	    pObjIndex->weight = 10;
	}
	else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_HEAD) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 11;
	    pObjIndex->value[3] = pObjIndex->level / 15;

	    pObjIndex->weight = 50;
	}
	else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_ABOUT) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 11;
	    pObjIndex->value[3] = pObjIndex->level / 15;

	    pObjIndex->weight = 20;
	}
	else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_BODY) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 10;
	    pObjIndex->value[3] = pObjIndex->level / 14;

	    pObjIndex->weight = 300;
	}
	else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_LEGS) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 10;
	    pObjIndex->value[3] = pObjIndex->level / 14;

	    pObjIndex->weight = 40;
	}
	else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_ARMS) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 10;
	    pObjIndex->value[3] = pObjIndex->level / 14;

	    pObjIndex->weight = 30;
	}
	else if ( IS_SET(pObjIndex->wear_flags, ITEM_WEAR_SHIELD) )
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = pObjIndex->value[2] =
	        1 + 3 * pObjIndex->level / 10;
	    pObjIndex->value[3] = pObjIndex->level / 14;

	    pObjIndex->weight = 200;
	}
	else
	{
	    pObjIndex->value[0] = pObjIndex->value[1] = 0;
	    pObjIndex->value[2] = pObjIndex->value[3] = 0;
	}

	break;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_food(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_FOOD )
    {
	send_to_char( "Object is not food.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "fullness")
    &&    str_prefix(arg, "hunger")
    &&    str_prefix(arg, "poison")) )
    {
	send_to_char( "Syntax: obj <vnum> food fullness <n>\n\r"
	              "        obj <vnum> food hunger <n>\n\r"
	              "        obj <vnum> food poison\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(arg, "fullness") )
    {
	int fullness;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The fullness must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	fullness = atoi( argument );
	if ( fullness < 0 )
	{
	    send_to_char( "The fullness must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] = fullness;
    }
    else if ( !str_prefix(arg, "hunger") )
    {
	int hunger;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The hunger must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	hunger = atoi( argument );
	if ( hunger < 0 )
	{
	    send_to_char( "The hunger must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = hunger;
    }
    else if ( !str_prefix(arg, "poison") )
    {
	if ( IS_SET(pObjIndex->value[3], (A)) )
	{
	    REMOVE_BIT(pObjIndex->value[3], (A) );
	    send_to_char( "It is no longer poisoned.\n\r", ch );
	}
	else
	{
	    SET_BIT(pObjIndex->value[3], (A) );
	    send_to_char( "It is now poisoned.\n\r", ch );
	}
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_liquid(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_DRINK_CON
    &&   pObjIndex->item_type != ITEM_FOUNTAIN )
    {
	send_to_char( "Object is not a liquid or fountain.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "type")
    &&    str_prefix(arg, "capacity")
    &&    str_prefix(arg, "contains")
    &&    str_prefix(arg, "poison")) )
    {
	send_to_char( "Syntax: obj <vnum> liquid type <liquid name>\n\r"
	              "        obj <vnum> liquid capacity <n>\n\r"
	              "        obj <vnum> liquid contains <n>\n\r"
	              "        obj <vnum> liquid poison\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(arg, "capacity") )
    {
	int capacity;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The capacity must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	capacity = atoi( argument );
	if ( capacity < 0 )
	{
	    send_to_char( "The capacity must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] = capacity;
    }
    else if ( !str_prefix(arg, "contains") )
    {
	int contains;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The amount contained must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	contains = atoi( argument );
	if ( contains < 0 )
	{
	    send_to_char( "The amount contained must be non-negative.\n\r", ch );
	    return FALSE;
	}
	if ( contains > pObjIndex->value[0] )
	{
	    send_to_char( "The amount contained must be less than the capacity.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = contains;
    }
    else if ( !str_prefix(arg, "type") )
    {
	int liquid;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the liquid type to what?\n\r", ch );
	    return FALSE;
	}
	
	liquid = liq_lookup( argument );
	
	if ( liquid == -1 )
	{
	    send_to_char( "That's not a valid liquid type.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[2] = liquid;
    }
    else if ( !str_prefix(arg, "poison") )
    {
	if ( IS_SET(pObjIndex->value[3], (A)) )
	{
	    REMOVE_BIT(pObjIndex->value[3], (A) );
	    send_to_char( "It is no longer poisoned.\n\r", ch );
	}
	else
	{
	    SET_BIT(pObjIndex->value[3], (A) );
	    send_to_char( "It is now poisoned.\n\r", ch );
	}
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_light(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_LIGHT )
    {
	send_to_char( "Object is a light source.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   str_prefix(arg, "duration") )
    {
	send_to_char( "Syntax: obj <vnum> light duration <n>\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(arg, "duration") )
    {
	int duration;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The duration must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	duration = atoi( argument );
	if ( duration < -1 )
	{
	    send_to_char( "The duration must be -1 or higher.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[2] = duration;
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_armor(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_ARMOR )
    {
	send_to_char( "Object is not armor.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   str_cmp(arg, "values") )
    {
	send_to_char( "Syntax: obj <vnum> armor values <pierce> <bash> <slash> <exotic>\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(arg, "values") )
    {
	int pierce, bash, slash, exotic;
	
	argument = one_argument( argument, arg );
	if ( !is_number(arg) )
	{
	    send_to_char( "The pierce-ac must be numeric.\n\r", ch );
	    return FALSE;
	}
	pierce = atoi( arg );
	if ( pierce < 0 )
	{
	    send_to_char( "The pierce ac must be non-negative.\n\r", ch );
	    return FALSE;
	}

	argument = one_argument( argument, arg );
	if ( !is_number(arg) )
	{
	    send_to_char( "The bash-ac must be numeric.\n\r", ch );
	    return FALSE;
	}
	bash = atoi( arg );
	if ( bash < 0 )
	{
	    send_to_char( "The bash ac must be non-negative.\n\r", ch );
	    return FALSE;
	}

	argument = one_argument( argument, arg );
	if ( !is_number(arg) )
	{
	    send_to_char( "The slash-ac must be numeric.\n\r", ch );
	    return FALSE;
	}
	slash = atoi( arg );
	if ( slash < 0 )
	{
	    send_to_char( "The slash ac must be non-negative.\n\r", ch );
	    return FALSE;
	}

	argument = one_argument( argument, arg );
	if ( !is_number(arg) )
	{
	    send_to_char( "The exotic-ac must be numeric.\n\r", ch );
	    return FALSE;
	}
	exotic = atoi( arg );
	if ( exotic < 0 )
	{
	    send_to_char( "The exotic ac must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] = pierce;
	pObjIndex->value[1] = bash;
	pObjIndex->value[2] = slash;
	pObjIndex->value[3] = exotic;
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_furniture(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_FURNITURE )
    {
	send_to_char( "Object is not a piece of furniture.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "people")
    &&    str_prefix(arg, "weight")
    &&    str_prefix(arg, "heal")
    &&    str_prefix(arg, "mana")
    &&    str_prefix(arg, "flags")) )
    {
	send_to_char( "Syntax: obj <vnum> furniture people <n>\n\r"
	              "        obj <vnum> furniture weight <n>\n\r"
	              "        obj <vnum> furniture heal <n>\n\r"
	              "        obj <vnum> furniture mana <n>\n\r"
	              "        obj <vnum> furniture flags <flags>\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(arg, "people") )
    {
	int people;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The number of people must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	people = atoi( argument );
	if ( people <= 0 )
	{
	    send_to_char( "The number of people must be positive.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] = people;
    }
    else if ( !str_prefix(arg, "weight") )
    {
	int weight;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The weight must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	weight = atoi( argument );
	if ( weight <= 0 )
	{
	    send_to_char( "The weight must be positive.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = weight;
    }
    else if ( !str_prefix(arg, "heal") )
    {
	int heal;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The heal modifier must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	heal = atoi( argument );
	if ( heal < -1 )
	{
	    send_to_char( "The heal modifier must be -1 or higher.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[3] = heal;
    }
    else if ( !str_prefix(arg, "mana") )
    {
	int mana;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The mana modifier must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	mana = atoi( argument );
	if ( mana < -1 )
	{
	    send_to_char( "The mana modifier must be -1 or higher.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[4] = mana;
    }
    else if ( !str_prefix(arg, "flags") )
    {
	long flags;
	int type;
	int pos;
	
	type = argument[0];
	if ( type == '=' || type == '-' || type == '+' )
	    argument++;
	    
	flags = 0;
	for ( ; ; )
	{
	    argument = one_argument( argument, arg );
	    if ( arg[0] == '\0' )
		break;
	
	    pos = flag_lookup( arg, furniture_flags );
	    if ( pos == 0 )
	    {
		sprintf( buf, "Bad flag %s.\n\r", arg );
		send_to_char( buf, ch );
		return FALSE;
	    }
	    
	    SET_BIT( flags, pos );
	}
	
	for ( pos = 0; furniture_flags[pos].name != NULL; pos++ )
	{
	    if ( !furniture_flags[pos].settable 
	    &&   IS_SET(flags, furniture_flags[pos].bit) )
	    {
		sprintf( buf, "Flag cannot be modified: %s.\n\r", 
		    furniture_flags[pos].name );
		send_to_char( buf, ch );
		return FALSE;
	    }
	}
	
	switch ( type )
	{
	case '+':
	    SET_BIT( pObjIndex->value[2], flags );
	    break;
	case '-':
	    REMOVE_BIT( pObjIndex->value[2], flags );
	    break;
	case '=':
	    pObjIndex->value[2] = flags;
	    break;
	default:
	    pObjIndex->value[2] = pObjIndex->value[2] ^ flags;
	}
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_use(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_WAND
    &&   pObjIndex->item_type != ITEM_STAFF )
    {
	send_to_char( "Object is not a wand or staff.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "spell")
    &&    str_prefix(arg, "charges")
    &&    str_prefix(arg, "level")
    &&    str_prefix(arg, "maximum")) )
    {
	send_to_char( "Syntax: obj <vnum> use spell <spell name>\n\r"
	              "        obj <vnum> use charges <n>\n\r"
	              "        obj <vnum> use maximum <n>\n\r"
	              "        obj <vnum> use level <n>\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(arg, "spell") )
    {
	int spell;

	spell = skill_lookup( argument );
	if ( spell == -1 )
	{
	    send_to_char( "Spell not found.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[3] = spell;
    }
    else if ( !str_prefix(arg, "maximum") )
    {
	int maximum;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The maximum charges contained must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	maximum = atoi( argument );
	if ( maximum < 0 )
	{
	    send_to_char( "The maximum charges contained must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = maximum;
    }
    else if ( !str_prefix(arg, "charges") )
    {
	int charges;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The current charges contained must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	charges = atoi( argument );
	if ( charges < 0 )
	{
	    send_to_char( "The current charges contained must be non-negative.\n\r", ch );
	    return FALSE;
	}
	if ( charges > pObjIndex->value[1] )
	{
	    send_to_char( "The current charges cannot be higher than the maximum.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[2] = charges;
    }
    else if ( !str_prefix(arg, "level") )
    {
	int level;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The level must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	level = atoi( argument );
	if ( level < 0 )
	{
	    send_to_char( "The level must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] = level;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_spell(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_PILL
    &&   pObjIndex->item_type != ITEM_POTION
    &&   pObjIndex->item_type != ITEM_SCROLL )
    {
	send_to_char( "Object is not a pill, potion or scroll.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "spells")
    &&    str_prefix(arg, "level")) )
    {
	send_to_char( "Syntax: obj <vnum> spell spells <spell list>\n\r"
	              "        obj <vnum> spell level <n>\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(arg, "spells") )
    {
	int spell;
	
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	{
	    send_to_char( "Set the spells to what?\n\r", ch );
	    return FALSE;
	}
	spell = skill_lookup( arg );
	if ( spell == -1 )
	{
	    send_to_char( "Spell not found.\n\r", ch );
	    return FALSE;
	}
	pObjIndex->value[1] = spell;
	
	argument = one_argument( argument, arg );
	if ( arg[0] != '\0' )
	{
	    spell = skill_lookup( arg );
	    if ( spell == -1 )
	    {
		send_to_char( "Spell not found.\n\r", ch );
		return FALSE;
	    }
	    pObjIndex->value[2] = spell;
	    
	    argument = one_argument( argument, arg );
	    if ( arg[0] != '\0' )
	    {
		spell = skill_lookup( arg );
		if ( spell == -1 )
		{
		    send_to_char( "Spell not found.\n\r", ch );
		    return FALSE;
		}
		pObjIndex->value[3] = spell;
	
		argument = one_argument( argument, arg );
		if ( arg[0] != '\0' )
		{
		    spell = skill_lookup( arg );
		    if ( spell == -1 )
		    {
			send_to_char( "Spell not found.\n\r", ch );
			return FALSE;
		    }
		    pObjIndex->value[4] = spell;
		}
	    }
	}
    }
    else if ( !str_prefix(arg, "level") )
    {
	int level;
	
	if ( !is_number(argument) )
	{
	    send_to_char( "The level must be numeric.\n\r", ch );
	    return FALSE;
	}
	
	level = atoi( argument );
	if ( level < 0 )
	{
	    send_to_char( "The level must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] = level;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_container(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_CONTAINER )
    {
	send_to_char( "Object is not a container.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "capacity")
    &&    str_prefix(arg, "weight")
    &&    str_prefix(arg, "multiplier")
    &&    str_prefix(arg, "key")
    &&    str_prefix(arg, "status")) )
    {
	send_to_char( "Syntax: obj <vnum> container capacity <n>\n\r"
	              "        obj <vnum> container weight <n>\n\r"
	              "        obj <vnum> container multiplier <n>\n\r"
	              "        obj <vnum> container key <n>\n\r"
	              "        obj <vnum> container status <closed|closeable|locked|pickproof>\n\r", ch );
	return FALSE;
    }

    if ( !str_prefix(arg, "capacity") )
    {
	int capacity;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the capacity to what?\n\r", ch );
	    return FALSE;
	}

        capacity = atoi( argument );
	if ( capacity < 0 )
	{
	    send_to_char( "The container capacity must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] =  capacity;
    }
    else if ( !str_prefix(arg, "weight") )
    {
	int weight;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the maximum weight to what?\n\r", ch );
	    return FALSE;
	}

        weight = atoi( argument );
	if ( weight < 0 )
	{
	    send_to_char( "The maximum weight must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[3] =  weight;
    }
    else if ( !str_prefix(arg, "multiplier") )
    {
	int multiplier;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the multiplier to what?\n\r", ch );
	    return FALSE;
	}

        multiplier = atoi( argument );
	if ( multiplier < 1 )
	{
	    send_to_char( "The multiplier must 1 or more.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[4] =  multiplier;
    }
    else if ( !str_prefix(arg, "key") )
    {
	int key;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the key to what?\n\r", ch );
	    return FALSE;
	}

        key = atoi( argument );
	if ( get_obj_index(key) == NULL && key != -1 && key != 0 )
	{
	    send_to_char( "Invalid key vnum.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[2] =  key;
    }
    else if ( !str_prefix(arg, "status") )
    {
	if ( !str_prefix(argument, "closed") )
	    pObjIndex->value[1] ^= CONT_CLOSED;
	else if ( !str_prefix(argument, "closeable") )
	    pObjIndex->value[1] ^= CONT_CLOSEABLE;
	else if ( !str_prefix(argument, "pickproof") )
	    pObjIndex->value[1] ^= CONT_PICKPROOF;
	else if ( !str_prefix(argument, "locked") )
	    pObjIndex->value[1] ^= CONT_LOCKED;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_weapon(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_WEAPON )
    {
	send_to_char( "Object is not a weapon.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "type")
    &&    str_prefix(arg, "damage")
    &&    str_prefix(arg, "message")
    &&    str_prefix(arg, "flags")) )
    {
	send_to_char( "Syntax: obj <vnum> weapon type <weapon type>\n\r"
	              "        obj <vnum> weapon damage <n>d<n>\n\r"
	              "        obj <vnum> weapon message <damage type>\n\r"
	              "        obj <vnum> weapon flags [+] <weapon flags>\n\r", ch );
	return FALSE;
    }

    if ( !str_prefix(arg, "type") )
    {
	int type;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the weapon type to what?\n\r", ch );
	    return FALSE;
	}

        type = weapon_type( argument );
	if ( type == WEAPON_EXOTIC && str_prefix(argument, "exotic") )
	{
	    send_to_char( "Invalid weapon type.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] =  type;
    }
    else if ( !str_prefix(arg, "damage") )
    {
	int dam_number, dam_type;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the damage to what?\n\r", ch );
	    return FALSE;
	}

	if ( !get_two_dice(argument, &dam_number, &dam_type) )
	{
	    send_to_char( "Invalid damage.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = dam_number;
	pObjIndex->value[2] = dam_type;
    }
    else if ( !str_prefix(arg, "message") )
    {
	int dam_type;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the damage message to what?\n\r", ch );
	    return FALSE;
	}

        dam_type = attack_lookup( argument );
	if ( dam_type == 0 )
	{
	    send_to_char( "Invalid damage type.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[3] =  dam_type;
    }
    else if ( !str_prefix(arg, "flags") )
    {
	long flags;
	int type;
	int pos;
	
	type = argument[0];
	if ( type == '=' || type == '-' || type == '+' )
	    argument++;
	    
	flags = 0;
	for ( ; ; )
	{
	    argument = one_argument( argument, arg );
	    if ( arg[0] == '\0' )
		break;
	
	    pos = flag_lookup( arg, weapon_flags );
	    if ( pos == 0 )
	    {
		sprintf( buf, "Bad flag %s.\n\r", arg );
		send_to_char( buf, ch );
		return FALSE;
	    }
	    
	    SET_BIT( flags, pos );
	}
	
	for ( pos = 0; weapon_flags[pos].name != NULL; pos++ )
	{
	    if ( !weapon_flags[pos].settable 
	    &&   IS_SET(flags, weapon_flags[pos].bit) )
	    {
		sprintf( buf, "Flag cannot be modified: %s.\n\r", 
		    weapon_flags[pos].name );
		send_to_char( buf, ch );
		return FALSE;
	    }
	}
	
	switch ( type )
	{
	case '+':
	    SET_BIT( pObjIndex->value[4], flags );
	    break;
	case '-':
	    REMOVE_BIT( pObjIndex->value[4], flags );
	    break;
	case '=':
	    pObjIndex->value[4] = flags;
	    break;
	default:
	    pObjIndex->value[4] = pObjIndex->value[4] ^ flags;
	}
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}

bool do_object_quiver(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_QUIVER )
    {
	send_to_char( "Object is not a quiver.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "arrows")
    &&    str_prefix(arg, "arrow_vnum")) )
    {
	send_to_char( "Syntax: obj <vnum> quiver arrows <number>\n\r"
	              "        obj <vnum> quiver arrow_vnum <vnum>\n\r", ch );
	return FALSE;
    }

    if ( !str_prefix(arg, "arrows") )
    {
	if ( argument[0] == '\0' )
	{
	    send_to_char( "How many arrows are in the quiver?\n\r", ch );
	    return FALSE;
	}

	pObjIndex->value[0] = atoi( argument );
    }

/*    else if ( !str_prefix(arg, "damage") )
    {
	int dam_number, dam_type;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the damage to what?\n\r", ch );
	    return FALSE;
	}

	if ( !get_two_dice(argument, &dam_number, &dam_type) )
	{
	    send_to_char( "Invalid damage.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = dam_number;
	pObjIndex->value[2] = dam_type;
    }
*/

    else if (!str_prefix(arg, "arrow_vnum") )
    {
      if ( argument[0] == '\0' )
      {
	pObjIndex->value[3] = OBJ_VNUM_ARROW;
      } 
      else
      {
      pObjIndex->value[3] = atoi( argument );
      }
    }
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}

bool do_object_arrow(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_ARROW )
    {
	send_to_char( "Object is not an arrow.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "damage") 
    &&   str_prefix(arg, "flags")) )
    {
	send_to_char( "Syntax: obj <vnum> arrow damage <n>d<n>\n\r", ch );
	return FALSE;
    }

    if ( !str_prefix(arg, "damage") )
    {
	int dam_number, dam_type;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the damage to what?\n\r", ch );
	    return FALSE;
	}

	if ( !get_two_dice(argument, &dam_number, &dam_type) )
	{
	    send_to_char( "Invalid damage.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = dam_number;
	pObjIndex->value[2] = dam_type;
    }
    else if ( !str_prefix(arg, "flags") )
    {
	long flags;
	int type;
	int pos;
	
	type = argument[0];
	if ( type == '=' || type == '-' || type == '+' )
	    argument++;
	    
	flags = 0;
	for ( ; ; )
	{
	    argument = one_argument( argument, arg );
	    if ( arg[0] == '\0' )
		break;
	
	    pos = flag_lookup( arg, weapon_flags );
	    if ( pos == 0 )
	    {
		sprintf( buf, "Bad flag %s.\n\r", arg );
		send_to_char( buf, ch );
		return FALSE;
	    }
	    
	    SET_BIT( flags, pos );
	}
	
	for ( pos = 0; weapon_flags[pos].name != NULL; pos++ )
	{
	    if ( !weapon_flags[pos].settable 
	    &&   IS_SET(flags, weapon_flags[pos].bit) )
	    {
		sprintf( buf, "Flag cannot be modified: %s.\n\r", 
		    weapon_flags[pos].name );
		send_to_char( buf, ch );
		return FALSE;
	    }
	}

	switch ( type )
	{
	case '+':
	    SET_BIT( pObjIndex->value[4], flags );
	    break;
	case '-':
	    REMOVE_BIT( pObjIndex->value[4], flags );
	    break;
	case '=':
	    pObjIndex->value[4] = flags;
	    break;
	default:
	    pObjIndex->value[4] = pObjIndex->value[4] ^ flags;
	}
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_poison(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    //char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_POISON )
    {
	send_to_char( "Object is not a poison.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "type")
    &&    str_prefix(arg, "duration")
     ))
    {
	send_to_char( "Syntax: obj <vnum> poison type <poison type>\n\r"
	              "        obj <vnum> poison duration <n>d<n>\n\r", ch );
	return FALSE;
    }

    if ( !str_prefix(arg, "type") )
    {
	int type;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the poison type to what?\n\r", ch );
	    return FALSE;
	}

   type = poison_type( argument );
	if ( type == -1 )
	{
	    send_to_char( "Invalid poison type.\n\r", ch );
		 send_to_char( "Valid types are: normal weakness nausea blindness bleeding\n\r"
		 					"                 paralysis hallucination coma madness wither death\n\r",
		 					ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] =  type;
    }

    else if ( !str_prefix(arg, "duration") )
    {
	int duration;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the duration to what?\n\r", ch );
	    return FALSE;
	}

   duration = atoi( argument );
	if ( duration == 0 )
	{
	    send_to_char( "A duration of 0 doesn't make any sense.\n\r", ch );
	    return FALSE;
	}
	
	if (( duration < -1 ) || ( duration > 127 ))
	{
	    send_to_char( "The duration must be -1 to 127.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = duration;
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_portal(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_PORTAL )
    {
	send_to_char( "Object is not a portal.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "charges")
    &&    str_prefix(arg, "flags")
    &&    str_prefix(arg, "exitflags")
    &&    str_prefix(arg, "key")
    &&    str_prefix(arg, "toroom")) )
    {
	send_to_char( "Syntax: obj <vnum> portal charges <n>\n\r"
	              "        obj <vnum> portal flags <flags>\n\r"
	              "        obj <vnum> portal exitflags <exitflags>\n\r"
	              "        obj <vnum> portal key <vnum>\n\r"
	              "        obj <vnum> portal toroom <vnum>\n\r", ch );
	return FALSE;
    }

    if ( !str_prefix(arg, "charges") )
    {
	int charges;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set number of charges to what?\n\r", ch );
	    return FALSE;
	}

        charges = atoi( argument );
	if ( charges < -1 )
	{
	    send_to_char( "The number of charges must be -1 or higher.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] =  charges;
    }
    else if ( !str_prefix(arg, "flags") )
    {
	long flags;
	int type;
	int pos;
	
	type = argument[0];
	if ( type == '=' || type == '-' || type == '+' )
	    argument++;
	    
	flags = 0;
	for ( ; ; )
	{
	    argument = one_argument( argument, arg );
	    if ( arg[0] == '\0' )
		break;
	
	    pos = flag_lookup( arg, portal_flags );
	    if ( pos == 0 )
	    {
		sprintf( buf, "Bad flag %s.\n\r", arg );
		send_to_char( buf, ch );
		return FALSE;
	    }
	    
	    SET_BIT( flags, pos );
	}
	
	for ( pos = 0; portal_flags[pos].name != NULL; pos++ )
	{
	    if ( !portal_flags[pos].settable 
	    &&   IS_SET(flags, portal_flags[pos].bit) )
	    {
		sprintf( buf, "Flag cannot be modified: %s.\n\r", 
		    portal_flags[pos].name );
		send_to_char( buf, ch );
		return FALSE;
	    }
	}
	
	switch ( type )
	{
	case '+':
	    SET_BIT( pObjIndex->value[2], flags );
	    break;
	case '-':
	    REMOVE_BIT( pObjIndex->value[2], flags );
	    break;
	case '=':
	    pObjIndex->value[2] = flags;
	    break;
	default:
	    pObjIndex->value[2] = pObjIndex->value[2] ^ flags;
	}
    }
    else if ( !str_prefix(arg, "key") )
    {
	int key;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the key to what?\n\r", ch );
	    return FALSE;
	}

        key = atoi( argument );
	if ( get_obj_index(key) == NULL && key != -1 && key != 0 )
	{
	    send_to_char( "Invalid key vnum.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[4] =  key;
    }
    else if ( !str_prefix(arg, "toroom") )
    {
	int toroom;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set the destination room to where?\n\r", ch );
	    return FALSE;
	}

        toroom = atoi( argument );
	if ( get_room_index(toroom) == NULL )
	{
	    send_to_char( "That destination does not exist.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[3] = toroom;
    }
    else if ( !str_prefix(arg, "exitflags") )
    {
	long flags;
	int type;
	int pos;
	
	type = argument[0];
	if ( type == '=' || type == '-' || type == '+' )
	    argument++;
	    
	flags = 0;
	for ( ; ; )
	{
	    argument = one_argument( argument, arg );
	    if ( arg[0] == '\0' )
		break;
	
	    pos = flag_lookup( arg, exit_flags );
	    if ( pos == 0 )
	    {
		sprintf( buf, "Bad flag %s.\n\r", arg );
		send_to_char( buf, ch );
		return FALSE;
	    }
	    
	    SET_BIT( flags, pos );
	}
	
	for ( pos = 0; exit_flags[pos].name != NULL; pos++ )
	{
	    if ( !exit_flags[pos].settable 
	    &&   IS_SET(flags, exit_flags[pos].bit) )
	    {
		sprintf( buf, "Flag cannot be modified: %s.\n\r", 
		    exit_flags[pos].name );
		send_to_char( buf, ch );
		return FALSE;
	    }
	}
	
	switch ( type )
	{
	case '+':
	    SET_BIT( pObjIndex->value[1], flags );
	    break;
	case '-':
	    REMOVE_BIT( pObjIndex->value[1], flags );
	    break;
	case '=':
	    pObjIndex->value[1] = flags;
	    break;
	default:
	    pObjIndex->value[1] = pObjIndex->value[4] ^ flags;
	}
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_focus(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_FOCUS )
    {
	send_to_char( "Object is not a focus.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   str_prefix(arg, "level") )
    {
	send_to_char( "Syntax: obj <vnum> focus level <n>\n\r", ch );
	return FALSE;
    }

    if ( !str_prefix(arg, "level") )
    {
	int level;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set number of focus power level to what?\n\r", ch );
	    return FALSE;
	}

        level = atoi( argument );
	if ( level <= 0 )
	{
	    send_to_char( "The focus power level must be positive.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] =  level;
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_money(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    
    if ( pObjIndex->item_type != ITEM_TREASURE )
    {
	send_to_char( "Object is not treasure.", ch );
	return FALSE;
    }
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "silver")
    &&    str_prefix(arg, "gold")) )
    {
	send_to_char( "Syntax: obj <vnum> money silver <n>\n\r"
	              "        obj <vnum> money gold <n>\n\r", ch );
	return FALSE;
    }

    if ( !str_prefix(arg, "silver") )
    {
	int silver;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set silver to what?\n\r", ch );
	    return FALSE;
	}

        silver = atoi( argument );
	if ( silver < 0 )
	{
	    send_to_char( "The silver must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[0] = silver;
    }
    else if ( !str_prefix(arg, "gold") )
    {
	int gold;
	
	if ( argument[0] == '\0' )
	{
	    send_to_char( "Set gold to what?\n\r", ch );
	    return FALSE;
	}

        gold = atoi( argument );
	if ( gold < 0 )
	{
	    send_to_char( "The gold must be non-negative.\n\r", ch );
	    return FALSE;
	}
	
	pObjIndex->value[1] = gold;
    }

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_limit( 
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    RESET_DATA *pReset;
    int limit;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Set object limit to what?\n\r", ch );
	return FALSE;
    }
    
    if ( !is_number(argument) )
    {
	send_to_char( "Invalid limit.\n\r", ch );
	return FALSE;
    }
    
    limit = atoi( argument );
    if ( limit < -1 )
    {
	send_to_char( "The limit must be -1 or greater.\n\r", ch );
	return FALSE;
    }
    
    for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
    {
	if ( (pReset->command == 'E' || pReset->command == 'G')
	&&   pReset->arg1 == pObjIndex->vnum )
	    pReset->arg2 = limit;
	else if ( pReset->command == 'P'
	&&   pReset->arg1 == pObjIndex->vnum )
	    pReset->arg2 = limit;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_resets( 
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    int lastMob, lastRoom;
    RESET_DATA *pReset;

    lastMob	= 0;
    lastRoom	= 0;
    
    for ( pReset = pArea->reset_first; pReset != NULL; pReset = pReset->next )
    {
	switch ( pReset->command )
	{
	case 'M':
	    lastMob	= pReset->arg1;
	    lastRoom	= pReset->arg3;
	    break;
	
	case 'E':
	    if ( pReset->arg1 != pObjIndex->vnum )
		break;
		
	    if ( lastMob == 0 )
		sprintf( buf, "Error: equip on an unknown mob.\n\r" );
	    else
	    {
		sprintf( buf, "Equiped on %d in room %d. Limit = %d\n\r",
		    lastMob, lastRoom, pReset->arg2 ); 
	    }
	    send_to_char( buf, ch );
	    break;
	
	case 'G':
	    if ( pReset->arg1 != pObjIndex->vnum )
		break;
		
	    if ( lastMob == 0 )
		sprintf( buf, "Error: give on an unknown mob.\n\r" );
	    else
	    {
		sprintf( buf, "Given to %d in room %d. Limit = %d\n\r",
		    lastMob, lastRoom, pReset->arg2 ); 
	    }
	    send_to_char( buf, ch );
	    break;
	
	case 'O':
	    if ( pReset->arg1 != pObjIndex->vnum )
		break;
		
	    sprintf( buf, "Placed in room %d.\n\r", pReset->arg3 );
	    send_to_char( buf, ch );
	    break;
	
	case 'P':
	    if ( pReset->arg1 != pObjIndex->vnum )
		break;
	
	    sprintf( buf, "Put in object %d %d times. Limit = %d\n\r",
		pReset->arg3, pReset->arg4, pReset->arg2 );
	    send_to_char( buf, ch );
	    break;
	
	default:
	    break;
	}
    }

    return FALSE;
}


bool do_object_fit(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    long flags;
    char type;
    int pos;

    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> fit +<flag list>\n\r"
	              "        obj <vnum> fit -<flag list>\n\r"
	              "        obj <vnum> fit =<flag list>\n\r"
	              "        obj <vnum> fit  <flag list>\n\r", ch );
	return FALSE;
    }
    
    type = argument[0];
    if (type == '=' || type == '-' || type == '+')
	argument++;

    flags = 0;
    for ( ; ; )
    {
	argument = one_argument( argument, arg );
	if ( arg[0] == '\0' )
	    break;
	
	pos = flag_lookup( arg, fit_flags );
	if ( pos == 0 )
	{
	    sprintf( buf, "Bad flag %s.\n\r", arg );
	    send_to_char( buf, ch );
	    return FALSE;
	}
	
	SET_BIT( flags, pos );
    }
    
    for ( pos = 0; fit_flags[pos].name != NULL; pos++ )
    {
        if ( !fit_flags[pos].settable && IS_SET(flags, fit_flags[pos].bit) )
	{
	    sprintf( buf, "Flag cannot be modified: %s.\n\r", 
	        fit_flags[pos].name );
	    send_to_char( buf, ch );
	    return FALSE;
	}
    }
    
    switch ( type )
    {
    case '+':
	SET_BIT( pObjIndex->fit_flags, flags );
	break;
    case '-':
	REMOVE_BIT( pObjIndex->fit_flags, flags );
	break;
    case '=':
	pObjIndex->fit_flags = flags;
	break;
    default:
	pObjIndex->fit_flags = pObjIndex->fit_flags ^ flags;
    }
    
    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_affect(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    AFFECT_DATA *paf;
    int flag, flag2;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> affect <affect flag>\n\r", ch );
	return FALSE;
    }
    
    flag = flag_lookup( argument, affect_flags );
    if ( flag == 0 )
    {
	flag2 = flag_lookup( argument, affect2_flags );
	if ( flag2 == 0 )
	{
	    send_to_char( "Invalid affect flag.\n\r", ch );
	    return FALSE;
	}

	for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
	{
	    if ( paf->where == TO_AFFECTS2 && paf->bitvector == flag )
		break;
	}

	if ( paf )
	{
	    if ( paf == pObjIndex->affected )
	    {
		pObjIndex->affected = paf->next;
		free_affect( paf );
	    }
	    else
	    {
		AFFECT_DATA *current;
	    
		for ( current = pObjIndex->affected;
		      current != NULL;
		      current = current->next )
		{
		    if ( current->next == paf )
		    {
			current->next = paf->next;
			free_affect( paf );
			break;
		    }
		}
	    }
	
	    send_to_char( "Affect removed.\n\r", ch );
	    return FALSE;
	}

	paf			= new_affect();
	paf->next		= pObjIndex->affected;
	pObjIndex->affected	= paf;

	paf->where		= TO_AFFECTS2;
	paf->type		= -1;
	paf->level		= pObjIndex->level;
	paf->duration		= -1;
	paf->location		= APPLY_NONE;
	paf->modifier		= 0;
	paf->bitvector		= flag2;
	paf->procNo		= 0;
	paf->flags		= 0;

	send_to_char( "OK.\n\r", ch );
	return TRUE;
    }

    for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->where == TO_AFFECTS && paf->bitvector == flag )
	    break;
    }

    if ( paf )
    {
	if ( paf == pObjIndex->affected )
	{
	    pObjIndex->affected = paf->next;
	    free_affect( paf );
	}
	else
	{
	    AFFECT_DATA *current;
	    
	    for ( current = pObjIndex->affected;
	          current != NULL;
	          current = current->next )
	    {
		if ( current->next == paf )
		{
		    current->next = paf->next;
		    free_affect( paf );
		    break;
		}
	    }
	}
	
	send_to_char( "Affect removed.\n\r", ch );
	return FALSE;
    }

    paf				= new_affect();
    paf->next			= pObjIndex->affected;
    pObjIndex->affected		= paf;

    paf->where			= TO_AFFECTS;
    paf->type			= -1;
    paf->level			= pObjIndex->level;
    paf->duration		= -1;
    paf->location		= APPLY_NONE;
    paf->modifier		= 0;
    paf->bitvector		= flag;
    paf->procNo			= 0;
    paf->flags			= 0;

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_immune(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    AFFECT_DATA *paf;
    int flag;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> immune <immune flag>\n\r", ch );
	return FALSE;
    }
    
    flag = flag_lookup( argument, imm_flags );
    if ( flag == 0 )
    {
	send_to_char( "Invalid immunity flag.\n\r", ch );
	return FALSE;
    }

    for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->where == TO_IMMUNE && paf->bitvector == flag )
	    break;
    }
    
    if ( paf )
    {
	if ( paf == pObjIndex->affected )
	{
	    pObjIndex->affected = paf->next;
	    free_affect( paf );
	}
	else
	{
	    AFFECT_DATA *current;
	    
	    for ( current = pObjIndex->affected;
	          current != NULL;
	          current = current->next )
	    {
		if ( current->next == paf )
		{
		    current->next = paf->next;
		    free_affect( paf );
		    break;
		}
	    }
	}
	
	send_to_char( "Immunity removed.\n\r", ch );
	return FALSE;
    }

    paf				= new_affect();
    paf->next			= pObjIndex->affected;
    pObjIndex->affected		= paf;

    paf->where			= TO_IMMUNE;
    paf->type			= -1;
    paf->level			= pObjIndex->level;
    paf->duration		= -1;
    paf->location		= APPLY_NONE;
    paf->modifier		= 0;
    paf->bitvector		= flag;
    paf->procNo			= 0;
    paf->flags			= 0;

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_resist(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    AFFECT_DATA *paf;
    int flag;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> resist <resistance flag>\n\r", ch );
	return FALSE;
    }
    
    flag = flag_lookup( argument, imm_flags );
    if ( flag == 0 )
    {
	send_to_char( "Invalid resistance flag.\n\r", ch );
	return FALSE;
    }

    for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->where == TO_RESIST && paf->bitvector == flag )
	    break;
    }
    
    if ( paf )
    {
	if ( paf == pObjIndex->affected )
	{
	    pObjIndex->affected = paf->next;
	    free_affect( paf );
	}
	else
	{
	    AFFECT_DATA *current;
	    
	    for ( current = pObjIndex->affected;
	          current != NULL;
	          current = current->next )
	    {
		if ( current->next == paf )
		{
		    current->next = paf->next;
		    free_affect( paf );
		    break;
		}
	    }
	}
	
	send_to_char( "Resistance removed.\n\r", ch );
	return FALSE;
    }

    paf				= new_affect();
    paf->next			= pObjIndex->affected;
    pObjIndex->affected		= paf;

    paf->where			= TO_RESIST;
    paf->type			= -1;
    paf->level			= pObjIndex->level;
    paf->duration		= -1;
    paf->location		= APPLY_NONE;
    paf->modifier		= 0;
    paf->bitvector		= flag;
    paf->procNo			= 0;
    paf->flags			= 0;

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_vuln(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    AFFECT_DATA *paf;
    int flag;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> vuln <vuln flag>\n\r", ch );
	return FALSE;
    }
    
    flag = flag_lookup( argument, imm_flags );
    if ( flag == 0 )
    {
	send_to_char( "Invalid vulnerability flag.\n\r", ch );
	return FALSE;
    }

    for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->where == TO_VULN && paf->bitvector == flag )
	    break;
    }
    
    if ( paf )
    {
	if ( paf == pObjIndex->affected )
	{
	    pObjIndex->affected = paf->next;
	    free_affect( paf );
	}
	else
	{
	    AFFECT_DATA *current;
	    
	    for ( current = pObjIndex->affected;
	          current != NULL;
	          current = current->next )
	    {
		if ( current->next == paf )
		{
		    current->next = paf->next;
		    free_affect( paf );
		    break;
		}
	    }
	}
	
	send_to_char( "Vulnerability removed.\n\r", ch );
	return FALSE;
    }

    paf				= new_affect();
    paf->next			= pObjIndex->affected;
    pObjIndex->affected		= paf;

    paf->where			= TO_VULN;
    paf->type			= -1;
    paf->level			= pObjIndex->level;
    paf->duration		= -1;
    paf->location		= APPLY_NONE;
    paf->modifier		= 0;
    paf->bitvector		= flag;
    paf->procNo			= 0;
    paf->flags			= 0;

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_skill(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg[MAX_INPUT_LENGTH];
    AFFECT_DATA *paf;
    int sn, modifier;
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> skill <skill name> <modifier>\n\r", ch );
	return FALSE;
    }

    argument = one_argument( argument, arg );
    
    sn = skill_lookup( arg );
    if ( sn == -1 )
    {
	send_to_char( "Invalid skill.\n\r", ch );
	return FALSE;
    }

    if ( argument[0] == '\0' || !is_number(argument) )
        return do_object_skill( ch, pArea, pObjIndex, "" );
    modifier = atoi( argument );

    for ( paf = pObjIndex->affected; paf != NULL; paf = paf->next )
    {
	if ( paf->where == TO_SKILL && paf->location == sn )
	    break;
    }
    
    if ( paf && modifier == 0 )
    {
	if ( paf == pObjIndex->affected )
	{
	    pObjIndex->affected = paf->next;
	    free_affect( paf );
	}
	else
	{
	    AFFECT_DATA *current;
	    
	    for ( current = pObjIndex->affected;
	          current != NULL;
	          current = current->next )
	    {
		if ( current->next == paf )
		{
		    current->next = paf->next;
		    free_affect( paf );
		    break;
		}
	    }
	}
	
	send_to_char( "Skill removed.\n\r", ch );
	return FALSE;
    }

    if ( paf == NULL )
    {
        paf                     = new_affect();
        paf->next		= pObjIndex->affected;
        pObjIndex->affected	= paf;
    }

    paf->where			= TO_SKILL;
    paf->type			= -1;
    paf->level			= pObjIndex->level;
    paf->duration		= -1;
    paf->location		= sn;
    paf->modifier		= modifier;
    paf->bitvector		= 0;
    paf->procNo			= 0;
    paf->flags			= 0;

    send_to_char( "OK.\n\r", ch );
    return TRUE;
}


bool do_object_program(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char arg1[MAX_INPUT_LENGTH];
    char arg2[MAX_INPUT_LENGTH];
    MPROG_LIST *pMprog;
    MPROG_CODE *prog;
    int trigger;

    argument = one_argument( argument, arg1 );
    argument = one_argument( argument, arg2 );

    if ( arg1[0] == '\0'
    ||   arg2[0] == '\0'
    ||   !is_number(arg2) )
    {
	send_to_char(
	    "Syntax: obj <vnum> program <trigger> <program vnum> [program argument]\n\r"
	    "        obj <vnum> program remove <number>\n\r",
	    ch );
	return FALSE;
    }
    
    if ( !str_cmp(arg1, "remove") )
    {
	MPROG_LIST *current, *remove;
	int number;

	number = atoi( arg2 );

	for ( remove = pObjIndex->mprogs; 
	      remove != NULL; 
	      remove = remove->next )
	{
	    if ( --number == 0 )
		break;
	}

	if ( remove != NULL )
	{
	    trigger = remove->trig_type;

	    if ( pObjIndex->mprogs == remove )
	    {
		pObjIndex->mprogs = pObjIndex->mprogs->next;
		free_mob_program( remove );
	    }
	    else
	    {
		for ( current = pObjIndex->mprogs; 
		      current != NULL; 
		      current = current->next )
		{
		    if ( current->next == NULL )
		    {
			send_to_char( "Obj program not found.\n\r", ch );
			return FALSE;
		    }

		    if ( current->next == remove )
		    {
			current->next = remove->next;
			free_mob_program( remove );

			break;
		    }
		}
	    }

	    /*REMOVE_BIT( pMobIndex->mprog_flags, trigger );*/

	    send_to_char( "OK.\n\r", ch );
	    return TRUE;
	}

	send_to_char( "That object doesn't have that many attached programs.\n\r", ch );
	return FALSE;
    }

    if ( (prog = get_mprog_index(atoi(arg2))) == NULL )
    {
	send_to_char( "Invalid program vnum.\n\r", ch );
	return FALSE;
    }

    trigger = flag_lookup( arg1, oprog_flags );
    if ( trigger == 0 )
    {
	char buf[MAX_STRING_LENGTH];
	int col;

	send_to_char( "Invalid trigger. Available triggers:\n\r", ch );

	for ( col=0, trigger=0; oprog_flags[trigger].name != NULL; trigger++ )
	{
	    sprintf( buf, "  %-10s", oprog_flags[trigger].name );
	    send_to_char( buf, ch );
	    
	    if ( ++col % 7 == 0 )
		send_to_char( "\n\r", ch );
	}

	if ( col % 7 == 0 )
	    send_to_char( "\n\r", ch );

	return FALSE;
    }

    pMprog		= new_mob_program();
    pMprog->next	= pObjIndex->mprogs;
    pObjIndex->mprogs	= pMprog;
    pMprog->trig_type	= trigger;
    SET_BIT( pObjIndex->mprog_flags, trigger );
    pMprog->vnum	= atoi( arg2 );
    free_string( pMprog->trig_phrase );
    pMprog->trig_phrase	= str_dup( argument );
    pMprog->code	= prog;

    return TRUE;
}


bool do_object_description(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    
    if ( argument[0] == '\0' )
    {
	send_to_char( "Syntax: obj <vnum> description <description>\n\r"
	              "        obj <vnum> description NULL\n\r", ch );
	return FALSE;
    }

    sprintf( buf, "'%s' %s", pObjIndex->name, argument );
    return do_object_extend( ch, pArea, pObjIndex, buf );
}


bool do_object_message(
  CHAR_DATA *ch, AREA_DATA *pArea, OBJ_INDEX_DATA *pObjIndex, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];
    int i;
    
    argument = one_argument( argument, arg );
    
    if ( arg[0] == '\0'
    ||   (str_prefix(arg, "list")
    &&    argument[0] == '\0') )
    {
	send_to_char( "Syntax: obj <vnum> message <tag> <text>\n\r"
	              "        obj <vnum> message <tag> NULL\n\r"
	              "        obj <vnum> message list\n\r", ch );
	return FALSE;
    }
    
    if ( !str_prefix(arg, "list") )
    {
	int col = 0;
	
	send_to_char( "Valid tags for this object type:\n\r", ch );
	
	for ( i = 0; !IS_NULLSTR(use_msgs[i].type); i++ )
	{
	    if ( use_msgs[i].item_type != 0
	    &&   use_msgs[i].item_type != pObjIndex->item_type )
		continue;

	    sprintf( buf, "%-15s  ", use_msgs[i].type );
	    send_to_char( buf, ch );
	    
	    if ( ++col % 4 == 0 )
		send_to_char( "\n\r", ch );
	}

	if ( col % 4 != 0 )
	    send_to_char( "\n\r", ch );
	    
	return FALSE;
    }
    
    for ( i = 0; !IS_NULLSTR(use_msgs[i].type); i++ )
    {
	if ( !str_prefix(arg, use_msgs[i].type) )
	{
	    USE_MSG_DATA *use_msg;
	    
	    if ( use_msgs[i].item_type != 0
	    &&   use_msgs[i].item_type != pObjIndex->item_type )
	    {
		send_to_char( "Invalid tag for this object type.\n\r", ch );
		return FALSE;
	    }
	    
	    use_msg = find_use_data( pObjIndex, use_msgs[i].tag );
	    
	    if ( !str_cmp(argument, "NULL") )
	    {
		if ( use_msg != NULL )
		    extract_obj_use_msg( pObjIndex, use_msgs[i].tag );
		
		send_to_char( "OK.\n\r", ch );
		return TRUE;
	    }
	    
	    if ( use_msg == NULL )
	    {
		use_msg = new_use_msg();
		use_msg->next = pObjIndex->use_msgs;
		pObjIndex->use_msgs = use_msg;
		
		use_msg->tag = use_msgs[i].tag;
	    }
	    
	    free_string( use_msg->msg );
	    use_msg->msg = str_dup( argument );
	    
	    send_to_char( "OK.\n\r", ch );
	    return TRUE;
	}
    }
    
    send_to_char( "Unknown tag specified, try using 'obj <vnum> message list'.\n\r", ch );
    return FALSE;
}
