/***************************************************************************
 *  Original Diku Mud copyright (C) 1990, 1991 by Sebastian Hammer,        *
 *  Michael Seifert, Hans Henrik St{rfeldt, Tom Madsen, and Katja Nyboe.   *
 *                                                                         *
 *  Merc Diku Mud improvments copyright (C) 1992, 1993 by Michael          *
 *  Chastain, Michael Quan, and Mitchell Tse.                              *
 *                                                                         *
 *  In order to use any part of this Merc Diku Mud, you must comply with   *
 *  both the original Diku license in 'license.doc' as well the Merc       *
 *  license in 'license.txt'.  In particular, you may not remove either of *
 *  these copyright notices.                                               *
 *                                                                         *
 *  Much time and thought has gone into this software and you are          *
 *  benefitting.  We hope that you share your changes too.  What goes      *
 *  around, comes around.                                                  *
 ***************************************************************************/

/***************************************************************************
*   ROM 2.4 is copyright 1993-1996 Russ Taylor                             *
*   ROM has been brought to you by the ROM consortium                      *
*       Russ Taylor (rtaylor@pacinfo.com)                                  *
*       Gabrielle Taylor (gtaylor@pacinfo.com)                             *
*       Brian Moore (rom@rom.efn.org)                                      *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

/***************************************************************************
*   THE LABYRINTH is copyright 1996-2002 Sherene Neil and William          *
*       Wallace                                                            *
*   THE LABYRINTH is brought to you by THE LABYRINTH Team:                 *
*   Sherene Wallace (sherene-dw@comcast.net)                               *
*   William Wallace (williamwallace@comcast.net)                           *
*                                                                          *
*   By using this code, you have agreed to follow the terms of the         *
*   ROM license, in the file Rom24/doc/rom.license                         *
***************************************************************************/

#if defined(macintosh)
#include <types.h>
#elif defined(WIN32)
#include <time.h>
#else
#include <sys/types.h>
#include <sys/time.h>
#endif
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <time.h>
#include "merc.h"


void do_sound( CHAR_DATA *ch, char *argument )
{
    char buf[MAX_STRING_LENGTH];
    char arg[MAX_INPUT_LENGTH];

    if ( IS_NPC(ch) )
    {
        send_to_char( "Only players may do that.\n\r", ch );
        return;
    }

    argument = one_argument( argument, arg );

    if ( arg[0] == '\0' )
    {
        sprintf( buf, "Sound settings:\n\r"
                      "  effects -- %s\n\r"
                      "  music   -- %s\n\r"
                      "Use: 'sound <effects|music> <on|off>' to update these settings.\n\r",
                 IS_SET(ch->act, PLR_SOUNDS) ? "on" : "off",
                 IS_SET(ch->act, PLR_MUSIC)  ? "on" : "off" );
        send_to_char( buf, ch );
        return;
    }

    if ( !str_prefix(arg, "effects") )
    {
        one_argument( argument, arg );
        if ( !str_cmp(arg, "on") )
            SET_BIT( ch->act, PLR_SOUNDS );
        else if ( !str_cmp(arg, "off") )
        {
            REMOVE_BIT( ch->act, PLR_SOUNDS );
            send_to_char( "!!SOUND(off)\n\r", ch );
        }
        else
        {
            send_to_char( "Specify either \"on\" or \"off\".\n\r", ch );
            return;
        }

        send_to_char( "OK.\n\r", ch );
        return;
    }
    else if ( !str_prefix(arg, "music") )
    {
        one_argument( argument, arg );
        if ( !str_cmp(arg, "on") )
            SET_BIT( ch->act, PLR_MUSIC );
        else if ( !str_cmp(arg, "off") )
        {
            REMOVE_BIT( ch->act, PLR_MUSIC );
            send_to_char( "!!MUSIC(off)\n\r", ch );
        }
        else
        {
            send_to_char( "Specify either \"on\" or \"off\".\n\r", ch );
            return;
        }

        send_to_char( "OK.\n\r", ch );
        return;
    }
    else
    {
        send_to_char( "Specify either \"effects\" or \"music\".\n\r", ch );
        return;
    }
}


void sound_to_char( const char *argument, CHAR_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *orig;

    if ( IS_NPC(ch) )
        if ( ch->desc != NULL && ch->desc->original != NULL )
            orig = ch->desc->original;
        else
            return;
    else
        orig = ch;

    
    if ( IS_SET(orig->act, PLR_SOUNDS) )
    {
        sprintf( buf, "!!SOUND(%s)\n\r", argument );
        send_to_char( buf, ch );
    }
}


void act_sound( const char *argument, CHAR_DATA *ch, CHAR_DATA *victim, int type )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *vch;

    if (ch == NULL || ch->in_room == NULL)
	return;

    sprintf( buf, "!!SOUND(%s)\n\r", argument );

    for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room )
    {
        CHAR_DATA *orig;

        if ( vch->desc == NULL )
            continue;

        if ( type == TO_CHAR && vch != ch )
            continue;
        if ( type == TO_ROOM && vch == ch )
            continue;
        if ( type == TO_VICT && (vch != victim || vch == ch) )
            continue;
        if ( type == TO_NOTVICT && (vch == ch || vch == victim) )
            continue;

        if ( IS_NPC(vch) )
        {
            if ( vch->desc->original != NULL )
                orig = vch->desc->original;
            else
                continue;
        }
        else
            orig = vch;

        if ( IS_SET(orig->act, PLR_SOUNDS) )
            send_to_char( buf, vch );
    }
}


void music_to_char( const char *argument, CHAR_DATA *ch )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *orig;

    if ( IS_NPC(ch) )
        if ( ch->desc != NULL && ch->desc->original != NULL )
            orig = ch->desc->original;
        else
            return;
    else
        orig = ch;

    
    if ( IS_SET(orig->act, PLR_MUSIC) )
    {
        sprintf( buf, "!!MUSIC(%s)\n\r", argument );
        send_to_char( buf, ch );
    }
}


void act_music( const char *argument, CHAR_DATA *ch, CHAR_DATA *victim, int type )
{
    char buf[MAX_STRING_LENGTH];
    CHAR_DATA *vch;

    if (ch == NULL || ch->in_room == NULL)
	return;

    sprintf( buf, "!!MUSIC(%s)\n\r", argument );

    for ( vch = ch->in_room->people; vch != NULL; vch = vch->next_in_room )
    {
        CHAR_DATA *orig;

        if ( vch->desc == NULL )
            continue;

        if ( type == TO_CHAR && vch != ch )
            continue;
        if ( type == TO_ROOM && vch == ch )
            continue;
        if ( type == TO_VICT && (vch != victim || vch == ch) )
            continue;
        if ( type == TO_NOTVICT && (vch == ch || vch == victim) )
            continue;

        if ( IS_NPC(vch) )
        {
            if ( vch->desc->original != NULL )
                orig = vch->desc->original;
            else
                continue;
        }
        else
            orig = vch;

        if ( IS_SET(orig->act, PLR_MUSIC) )
            send_to_char( buf, vch );
    }
}
